@@ -112,6 +112,10 @@
 #define UDM_MATCH_NUMERIC_LT	7
 #define UDM_MATCH_NUMERIC_GT	8
 
+/* Case sensitivity */
+#define UDM_CASE_SENSITIVE      0
+#define UDM_CASE_INSENSITIVE    1
+
 /* Indexer return codes */
 #define UDM_OK			0
 #define UDM_ERROR		1
@@ -120,14 +124,15 @@
 
 
 /* Flags for indexing */
-#define UDM_FLAG_REINDEX	1
-#define UDM_FLAG_SORT_EXPIRED	2
-#define UDM_FLAG_SORT_HOPS	4
-#define UDM_FLAG_ADD_SERV	8
-#define UDM_FLAG_SPELL		16
-#define UDM_FLAG_LOAD_LANGMAP	32
-#define UDM_FLAG_DONTSORT_SEED  64
-#define UDM_FLAG_ADD_SERVURL	128
+#define UDM_FLAG_REINDEX           1
+#define UDM_FLAG_SORT_EXPIRED      2
+#define UDM_FLAG_SORT_HOPS         4
+#define UDM_FLAG_ADD_SERV          8
+#define UDM_FLAG_SPELL	          16
+#define UDM_FLAG_LOAD_LANGMAP	    32
+#define UDM_FLAG_DONTSORT_SEED    64
+#define UDM_FLAG_ADD_SERVURL	   128
+#define UDM_FLAG_DONT_ADD_TO_DB  256
 
 
 /* URLFile actions */
@@ -258,11 +263,25 @@ typedef struct udm_stopword_struct {
 	char		*lang;
 } UDM_STOPWORD;
 
-typedef struct {
-	size_t		nstopwords;
-	UDM_STOPWORD	*StopWord;
+#define UDM_STOPLIST_LANGLEN 32
+#define UDM_STOPLIST_CSETLEN 32
+#define UDM_STOPLIST_FILELEN 128
+
+typedef struct
+{
+  size_t nstopwords;
+  UDM_STOPWORD	*StopWord;
+  char  lang[UDM_STOPLIST_LANGLEN];
+  char  cset[UDM_STOPLIST_CSETLEN];
+  char  fname[UDM_STOPLIST_FILELEN];
 } UDM_STOPLIST;
 
+typedef struct
+{
+  size_t nitems;
+  UDM_STOPLIST *Item;
+} UDM_STOPLISTLIST;
+
 /*****************************************************/
 
 /** Words parameters */
@@ -406,10 +425,14 @@ typedef struct {
 } UDM_SPIDERPARAM;
 
 /*****************************************************************/
+
+#define UDM_MATCH_FLAG_SKIP_OPTIMIZATION 1
+
 typedef struct {
 	int		match_type;
 	int		nomatch;
 	int		case_sense;
+	int		flags; /* e.g. skip normalizaton */
 	char		*section;
 	char		*pattern;
 	size_t          pattern_length;
@@ -701,12 +724,29 @@ typedef struct {
 	char	*s;
 } UDM_SYNONYM;
 
+#define UDM_SYNONYM_LANGLEN 32
+#define UDM_SYNONYM_CSETLEN 32
+#define UDM_SYNONYM_FILELEN 128
+#define UDM_SYNONYM_FMT_TEXT 0
+#define UDM_SYNONYM_FMT_HASH 1
+
 typedef struct {
 	size_t		nsynonyms;
 	size_t		msynonyms;
 	UDM_SYNONYM	*Synonym;
+        char  lang[UDM_SYNONYM_LANGLEN];
+        char  cset[UDM_SYNONYM_CSETLEN];
+        char  fname[UDM_SYNONYM_FILELEN];
 } UDM_SYNONYMLIST;
 
+
+typedef struct
+{
+  size_t nitems;
+  UDM_SYNONYMLIST *Item;
+} UDM_SYNONYMLISTLIST;
+
+
 typedef struct udm_chinaword_struct {
   int *word;
   int  freq;
@@ -827,8 +867,8 @@ typedef struct udm_config_struct {
 	
 	UDM_LANGMAPLIST	LangMaps;	/**< For lang+charset quesser   */
 	UDM_ROBOTS	Robots;		/**< robots.txt information     */
-	UDM_SYNONYMLIST	Synonyms;	/**< Synonims list              */
-	UDM_STOPLIST	StopWords;	/**< Stopwords list             */
+	UDM_SYNONYMLISTLIST Synonym;    /**< Synonims list              */
+	UDM_STOPLISTLIST StopWord;      /**< Stopwords list             */
 	UDM_PARSERLIST	Parsers;	/**< External  parsers          */
 	UDM_DBLIST	dbl;		/**< Searchd addresses	      */
 	UDM_HOSTLIST	Hosts;		/**< Resolve cache              */
@@ -872,7 +912,7 @@ typedef struct udm_indexer_struct{
 	int		freeme;		/**< whenever it was allocated    */
 	int		handle;		/**< Handler for threaded version */
 	time_t		start_time;	/**< Time of allocation, for stat */
-	unsigned long long nbytes;	/**< Number of bytes downloaded   */
+	udm_uint8   nbytes;     /**< Number of bytes downloaded   */
 	size_t		ndocs;		/**< Number of documents          */
         size_t          nsleepsecs;     /**> Number of sleep seconds      */
 	int		flags;		/**< Callback function to request action*/
@@ -1010,7 +1050,8 @@ enum udm_indcmd
   UDM_IND_REWRITEURL,
   UDM_IND_HASHSPELL,
   UDM_IND_DUMPSPELL,
-  UDM_IND_REWRITELIMITS
+  UDM_IND_REWRITELIMITS,
+  UDM_IND_DUMPCONF
 };
 
 #endif
