@@ -85,6 +85,7 @@ extern __C_LINK int __UDMCALL UdmTargets(UDM_AGENT *Indexer);
 
 
 extern __C_LINK UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *);
+extern __C_LINK UDM_RESULT * __UDMCALL UdmFind2(UDM_AGENT *, const char *query);
 extern __C_LINK int __UDMCALL UdmClearDatabase(UDM_AGENT *A);
 extern __C_LINK int __UDMCALL UdmStatAction(UDM_AGENT *A, UDM_STATLIST *S);
 
@@ -130,6 +131,7 @@ extern int UdmCloneListSQL(UDM_AGENT * Indexer, UDM_DOCUMENT *Doc, UDM_RESULT *R
 extern int UdmTargetsSQL(UDM_AGENT *Indexer, UDM_DB *db);
 extern int UdmQueryCachePutSQL(UDM_AGENT *Indexer, UDM_RESULT *Res, UDM_DB *db);
 extern int UdmQueryCacheGetSQL(UDM_AGENT *Indexer, UDM_RESULT *Res, UDM_DB *db);
+extern int UdmStoreHrefsSQL(UDM_AGENT *Indexer);
 #endif
 
 /* MultiCache stuff. */
