@@ -15,6 +15,6 @@
  
 */
 
-extern char * UdmParseDate(const char * datestring);
+extern char * UdmDateParse(const char * datestring);
 
 #endif
