@@ -36,7 +36,7 @@ extern void UdmGroupByURL2(UDM_AGENT *Agent,
                            UDM_URLSCORELIST *ScoreList);
 
 extern int  UdmPrepare(UDM_AGENT *query,UDM_RESULT *res);
-extern __C_LINK int __UDMCALL UdmParseQueryString(UDM_AGENT * Agent,UDM_VARLIST * vars,char * query_string);
+extern __C_LINK int __UDMCALL UdmParseQueryString(UDM_AGENT * Agent,UDM_VARLIST * vars, const char * query_string);
 
 extern char *UdmHlConvert(UDM_WIDEWORDLIST *L,const char * src, UDM_CHARSET * lcs, UDM_CHARSET * bcs);
 extern size_t UdmHlConvertExt(char *dst, size_t dstlen,
