@@ -24,4 +24,9 @@ extern void           UdmStopListSort(UDM_STOPLIST *);
 extern __C_LINK int __UDMCALL UdmStopListLoad(UDM_ENV *,const char *fname);
 extern int            UdmStopListAdd(UDM_STOPLIST *,UDM_STOPWORD *);
 
+extern void UdmStopListListInit(UDM_STOPLISTLIST *Lst);
+extern void UdmStopListListFree(UDM_STOPLISTLIST *Lst);
+extern int  UdmStopListListAdd(UDM_STOPLISTLIST *Lst, UDM_STOPLIST *Item);
+extern UDM_STOPWORD *UdmStopListListFind(UDM_STOPLISTLIST *Lst,
+                                         const char *word, const char *lang);
 #endif
