@@ -22,6 +22,13 @@ extern void UdmSynonymListInit(UDM_SYNONYMLIST * List);
 extern __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename);
 extern void UdmSynonymListFree(UDM_SYNONYMLIST * List);
 extern __C_LINK void __UDMCALL UdmSynonymListSort(UDM_SYNONYMLIST * List);
-extern UDM_WIDEWORDLIST * UdmSynonymListFind(const UDM_SYNONYMLIST * List,UDM_WIDEWORD * wword);
+
+extern void UdmSynonymListListInit(UDM_SYNONYMLISTLIST *L);
+extern void UdmSynonymListListFree(UDM_SYNONYMLISTLIST *L);
+extern int  UdmSynonymListListAdd(UDM_SYNONYMLISTLIST *L, UDM_SYNONYMLIST *I);
+extern UDM_WIDEWORDLIST *UdmSynonymListListFind(const UDM_SYNONYMLISTLIST *List,
+                                                UDM_WIDEWORD *wword);
+extern __C_LINK void __UDMCALL UdmSynonymListListSortItems(UDM_SYNONYMLISTLIST * List);
+
 
 #endif
