@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -131,6 +131,8 @@ typedef struct
 
 struct udm_conv_st;
 
+struct udm_unidata_st;
+
 typedef struct udm_cset_st
 {
   int id;
@@ -138,9 +140,13 @@ typedef struct udm_cset_st
                const unsigned char *s,const unsigned char *e);
   int (*wc_mb)(struct udm_conv_st *conv, struct udm_cset_st *cs, int *wc,
                unsigned char *s,unsigned char *e);
-  void (*lcase)(struct udm_cset_st *cs, char *str, size_t bytelen);
+  void (*lcase)(struct udm_unidata_st *, struct udm_cset_st *cs, char *str, size_t bytelen);
+  char *(*septoken)(struct udm_unidata_st *, struct udm_cset_st *cs,
+                    const char *str, const char *strend,
+                    const char **last, int *ctype0);
   const char * name;
   int family;
+  unsigned char  *ctype;
   unsigned short *tab_to_uni;
   UDM_UNI_IDX    *tab_from_uni;
 } UDM_CHARSET;
@@ -149,13 +155,9 @@ typedef struct udm_conv_st
 {
   UDM_CHARSET    *from;
   UDM_CHARSET    *to;
-  int            flags;
-  size_t         ibytes;
-  size_t         obytes;
-  size_t         icodes;
-  size_t         ocodes;
-  int            istate;
-  int            ostate;
+  char           flags;
+  char           istate;
+  char           ostate;
 } UDM_CONV;
 
 
@@ -210,6 +212,13 @@ typedef struct
 /* Output buffer in uni2xxx is too small    */
 #define UDM_CHARSET_TOOSMALL  -1
 
+/*
+  "Unicode value was returned from cache"
+  For character sets like tscii, when one
+  native code corresponds to several Unicode characters:
+  8C -> U+0BE8 U+0BB7 U+0B82
+*/
+#define UDM_CHARSET_CACHEDUNI -100
 
 /* Character types */
 #define UDM_UNI_SEPAR    0
@@ -224,6 +233,20 @@ int udm_mb_wc_utf8(UDM_CONV *conv, UDM_CHARSET *c,int *pwc,
 extern __C_LINK int __UDMCALL udm_wc_mb_utf8(UDM_CONV *conv, UDM_CHARSET *c,int *wc, 
                                              unsigned char *s, unsigned char *e);
 
+extern __C_LINK int __UDMCALL udm_mb_wc_latin1(UDM_CONV *conv, UDM_CHARSET *c,int *pwc,
+                                               const unsigned char *s, 
+                                               const unsigned char *e);
+extern __C_LINK int __UDMCALL udm_wc_mb_latin1(UDM_CONV *conv, UDM_CHARSET *c,int *wc,
+                                               unsigned char *s, unsigned char *e);
+
+
+extern __C_LINK int __UDMCALL udm_mb_wc_ascii(UDM_CONV *conv, UDM_CHARSET *c,int *pwc,
+                                              const unsigned char *s, 
+                                              const unsigned char *e);
+extern __C_LINK int __UDMCALL udm_wc_mb_ascii(UDM_CONV *conv, UDM_CHARSET *c,int *wc,
+                                              unsigned char *s, unsigned char *e);
+
+
 extern __C_LINK int __UDMCALL udm_mb_wc_8bit(UDM_CONV *conv, UDM_CHARSET *c,int *pwc,
                                              const unsigned char *s, 
                                              const unsigned char *e);
