@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -171,7 +171,7 @@ extern int   UdmBuild(char * path, int mode);
 extern int UdmSetEnv(const char * name,const char * value);
 extern void UdmUnsetEnv(const char * name);
 
-extern void UdmUniRemoveDoubleSpaces(int * ustr);
+extern size_t UdmUniRemoveDoubleSpaces(int *ustr);
 extern void UdmUniPrint(int * ustr);
 
 extern ssize_t UdmRecvall(int s, void *buf, size_t len);
@@ -196,12 +196,12 @@ typedef struct dstr_struct {
 extern UDM_DSTR *UdmDSTRInit (UDM_DSTR *dstr, size_t size_page);
 extern void UdmDSTRFree (UDM_DSTR *dstr);
 extern size_t UdmDSTRAppend (UDM_DSTR *dstr, const char *data, size_t size_data);
+extern size_t UdmDSTRAppendHex(UDM_DSTR *dstr, const char *s, size_t slen);
 extern size_t UdmDSTRAppendSTR (UDM_DSTR *dstr, const char *data);
 extern int UdmDSTRAppendf (UDM_DSTR *dstr, const char *fmt, ...);
 extern void UdmDSTRReset (UDM_DSTR *dstr);
 extern int UdmDSTRAlloc (UDM_DSTR *dstr, size_t size_data);
 extern int UdmDSTRRealloc (UDM_DSTR *dstr, size_t size_data);
-extern void UdmSoundex(UDM_CHARSET *cs, char *dst, const char *src, size_t srclen);
 
 /********************************** zint4 stuff ******************************/
 typedef struct st_udm_zint4_state
@@ -216,4 +216,21 @@ extern void udm_zint4(UDM_ZINT4_STATE *state, int next);
 extern void udm_zint4_finalize(UDM_ZINT4_STATE *state);
 extern int udm_dezint4(const char *buf, int4 *array, int buf_len);
 /**************************** end of zint4 stuff *****************************/
+
+extern int udm_l1tolower[256];
+
+#ifdef HAVE_DEBUG
+extern void *UdmBSearch(const void *key, const void *base, size_t nmemb,
+                         size_t size,
+                         int (*compar)(const void *, const void *));
+extern void UdmSort(void *base, size_t nmemb, size_t size,
+                    int(*compar)(const void *, const void *));
+#else
+#define UdmBSearch bsearch
+#define UdmSort   qsort
+#endif
+
+#define UDM_MIN(x,y)  ((x) < (y) ? (x) : (y))
+#define UDM_MAX(x,y)  ((x) > (y) ? (x) : (y))
+
 #endif /* _UDM_UTILS_H */
