@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ extern int UdmVarType(UDM_VAR *);
 
 extern int UdmVarListAddEnviron(UDM_VARLIST *vars, const char *name);
 
+extern int UdmVarAppendStrn(UDM_VAR *var, const char *val, size_t len);
 extern __C_LINK int __UDMCALL UdmVarListAddStr(UDM_VARLIST * vars,const char * name, const char * val);
 extern int UdmVarListAddQueryStr(UDM_VARLIST * vars,const char * name, const char * val);
 extern int UdmVarListAddInt(UDM_VARLIST * vars,const char * name, int val);
@@ -44,6 +45,7 @@ extern int UdmVarListInvokeMethod(UDM_VARLIST *Lst,
 extern int UdmVarListAddLst(UDM_VARLIST * Lst,UDM_VARLIST *Src, const char *name, const char *w);
 extern int UdmVarListInsLst(UDM_VARLIST * Lst,UDM_VARLIST *Src, const char *name, const char *w);
 extern int UdmVarListReplaceLst(UDM_VARLIST * Lst,UDM_VARLIST *Src, const char *name, const char *w);
+extern int UdmVarListMerge(UDM_VARLIST * Lst,UDM_VARLIST *Src1, UDM_VARLIST *Src2);
 
 extern __C_LINK int __UDMCALL UdmVarListReplaceStr(UDM_VARLIST * vars,const char * name,const char * val);
 extern __C_LINK int __UDMCALL UdmVarListReplaceInt(UDM_VARLIST * vars,const char * name,int val);
