@@ -271,7 +271,9 @@ static int add_srv(void *Cfg, size_t ac,char **av)
   C->Srv->command = 'S';
   C->Srv->ordre = ++C->ordre;
   C->Srv->Match.nomatch=0;
-  C->Srv->Match.case_sense = 1;
+  C->Srv->Match.case_sense = UDM_CASE_INSENSITIVE;
+  C->Srv->Match.flags= (C->flags & UDM_FLAG_DONT_ADD_TO_DB) ?
+                       UDM_MATCH_FLAG_SKIP_OPTIMIZATION : 0;
   UdmVarListReplaceStr(&C->Srv->Vars, "Method", "Allow");
 
   if(!strcasecmp(av[0],"Server"))
@@ -297,9 +299,9 @@ static int add_srv(void *Cfg, size_t ac,char **av)
     else
     if(UDM_METHOD_UNKNOWN!=(o=UdmMethod(av[i])))UdmVarListReplaceStr(&C->Srv->Vars,"Method",av[i]);
     else
-    if(!strcasecmp(av[i],"nocase"))C->Srv->Match.case_sense=0;
+    if(!strcasecmp(av[i],"nocase"))C->Srv->Match.case_sense= UDM_CASE_SENSITIVE;
     else
-    if(!strcasecmp(av[i],"case"))C->Srv->Match.case_sense=1;
+    if(!strcasecmp(av[i],"case"))C->Srv->Match.case_sense= UDM_CASE_INSENSITIVE;
     else
     if(!strcasecmp(av[i],"match"))C->Srv->Match.nomatch=0;
     else
@@ -329,7 +331,7 @@ static int add_srv(void *Cfg, size_t ac,char **av)
     sprintf(Conf->errstr,"too few argiments in '%s' command", av[0]);
     return UDM_ERROR;
   }
-  if(UDM_OK != UdmServerAdd(Indexer, C->Srv))
+  if(UDM_OK != UdmServerAdd(Indexer, C->Srv, C->flags))
   {
     char * s_err;
     s_err = (char*)UdmStrdup(Conf->errstr);
@@ -399,7 +401,10 @@ static int add_alias(void *Cfg, size_t ac,char **av)
   
   UdmMatchInit(&Alias);
   Alias.match_type=UDM_MATCH_BEGIN;
-  
+  Alias.case_sense= UDM_CASE_INSENSITIVE;
+  Alias.flags= (C->flags & UDM_FLAG_DONT_ADD_TO_DB) ?
+                UDM_MATCH_FLAG_SKIP_OPTIMIZATION : 0;
+
   for(i=1; i<ac; i++)
   {
     if(!strcasecmp(av[i],"regex"))
@@ -407,9 +412,9 @@ static int add_alias(void *Cfg, size_t ac,char **av)
     else if(!strcasecmp(av[i],"regexp"))
       Alias.match_type=UDM_MATCH_REGEX;
     else if(!strcasecmp(av[i],"case"))
-      Alias.case_sense=1;
+      Alias.case_sense= UDM_CASE_INSENSITIVE;
     else if(!strcasecmp(av[i],"nocase"))
-      Alias.case_sense=0;
+      Alias.case_sense= UDM_CASE_SENSITIVE;
     else if(!Alias.pattern)
     {
       Alias.pattern=av[i];
@@ -452,14 +457,16 @@ static int add_filter(void *Cfg, size_t ac,char **av)
   
   UdmMatchInit(&M);
   M.match_type=UDM_MATCH_WILD;
-  M.case_sense=1;
-  
+  M.case_sense= UDM_CASE_INSENSITIVE;
+  M.flags= (C->flags & UDM_FLAG_DONT_ADD_TO_DB) ?
+            UDM_MATCH_FLAG_SKIP_OPTIMIZATION : 0;
+
   C->ordre++;
   for(i=1; i<ac ; i++)
   {
-    if(!strcasecmp(av[i],"case"))M.case_sense=1;
+    if(!strcasecmp(av[i],"case"))M.case_sense= UDM_CASE_INSENSITIVE;
     else
-    if(!strcasecmp(av[i],"nocase"))M.case_sense=0;
+    if(!strcasecmp(av[i],"nocase"))M.case_sense= UDM_CASE_SENSITIVE;
     else
     if(!strcasecmp(av[i],"regex"))M.match_type=UDM_MATCH_REGEX;
     else
@@ -501,14 +508,16 @@ static int add_section_filter(void *Cfg, size_t ac,char **av)
   
   UdmMatchInit(&M);
   M.match_type=UDM_MATCH_WILD;
-  M.case_sense=1;
-  
+  M.case_sense= UDM_CASE_INSENSITIVE;
+  M.flags= (C->flags & UDM_FLAG_DONT_ADD_TO_DB) ?
+            UDM_MATCH_FLAG_SKIP_OPTIMIZATION : 0;
+
   C->ordre++;
   for(i=1; i<ac ; i++)
   {
-    if(!strcasecmp(av[i],"case"))M.case_sense=1;
+    if(!strcasecmp(av[i],"case"))M.case_sense= UDM_CASE_INSENSITIVE;
     else
-    if(!strcasecmp(av[i],"nocase"))M.case_sense=0;
+    if(!strcasecmp(av[i],"nocase"))M.case_sense= UDM_CASE_SENSITIVE;
     else
     if(!strcasecmp(av[i],"regex"))M.match_type=UDM_MATCH_REGEX;
     else
@@ -568,7 +577,10 @@ static int add_type(void *Cfg, size_t ac,char **av)
   
   UdmMatchInit(&M);
   M.match_type=UDM_MATCH_WILD;
-  
+  M.case_sense= UDM_CASE_INSENSITIVE;
+  M.flags= (C->flags & UDM_FLAG_DONT_ADD_TO_DB) ?
+            UDM_MATCH_FLAG_SKIP_OPTIMIZATION : 0;
+
   for (i=1; i<ac; i++)
   {
     if(!strcasecmp(av[i],"regex"))M.match_type=UDM_MATCH_REGEX;
@@ -577,9 +589,13 @@ static int add_type(void *Cfg, size_t ac,char **av)
     else
     if(!strcasecmp(av[i],"string"))M.match_type=UDM_MATCH_WILD;
     else
-    if(!strcasecmp(av[i],"case"))M.case_sense=1;
+    if(!strcasecmp(av[i],"match"))M.nomatch= 0;
     else
-    if(!strcasecmp(av[i],"nocase"))M.case_sense=0;
+    if(!strcasecmp(av[i],"nomatch"))M.nomatch= 1;
+    else
+    if(!strcasecmp(av[i],"case"))M.case_sense= UDM_CASE_INSENSITIVE;
+    else
+    if(!strcasecmp(av[i],"nocase"))M.case_sense= UDM_CASE_SENSITIVE;
     else
     if(!M.arg)
       M.arg=av[i];
@@ -690,8 +706,10 @@ static int add_section(void *Cfg, size_t ac,char **av)
 
     UdmMatchInit(&M);
     M.match_type= UDM_MATCH_REGEX;
-    M.case_sense= 1;
+    M.case_sense= UDM_CASE_INSENSITIVE;
     M.section= S.name;
+    M.flags= (C->flags & UDM_FLAG_DONT_ADD_TO_DB) ?
+              UDM_MATCH_FLAG_SKIP_OPTIMIZATION : 0;
 
     switch (ac)
     {
@@ -1540,6 +1558,16 @@ __C_LINK int __UDMCALL UdmEnvAddLine(UDM_CFG *C,char *str)
 }
 
 
+__C_LINK int __UDMCALL UdmAgentAddLine(UDM_AGENT *Agent, const char *line)
+{
+  UDM_CFG Cfg;
+  char str[1024];
+  bzero((void*) &Cfg, sizeof(Cfg));
+  Cfg.Indexer= Agent;
+  udm_snprintf(str, sizeof(str) - 1, line);
+  return UdmEnvAddLine(&Cfg, str);
+}
+
 static int EnvLoad(UDM_CFG *Cfg,const char *cname)
 {
   char  *str0 = NULL;  /* Unsafe copy - will be used in strtok  */
@@ -1656,3 +1684,399 @@ freeex:
   UdmServerFree(&Srv);
   return rc;
 }
+
+
+
+static size_t
+UdmMatchToStr(char *str, size_t size, const UDM_MATCH *M, const char *cmd)
+{
+  if (cmd)
+    return udm_snprintf(str, size, "%s %s%s%s \"%s\" \"%s\"",
+                        cmd,
+                        M->match_type == UDM_MATCH_REGEX ? " regex" : "",
+                        M->nomatch ? " nomatch" : "",
+                        M->case_sense ? "" : " NoCase",
+                        M->arg, M->pattern);
+  else
+    return udm_snprintf(str, size, "%s %s%s%s \"%s\"",
+                        M->arg,
+                        M->match_type == UDM_MATCH_REGEX ? " regex" : "",
+                        M->nomatch ? " nomatch" : "",
+                        M->case_sense ? "" : " NoCase",
+                        M->pattern);
+
+}
+
+
+static int
+UdmMatchListPrint(FILE *f, UDM_MATCHLIST *L, const char *cmd)
+{
+  size_t i;
+  char str[128];
+  for (i= 0; i < L->nmatches; i++)
+  {
+    UDM_MATCH *M= &L->Match[i];
+    UdmMatchToStr(str, sizeof(str), M, cmd);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmSectionListPrint(FILE *f, UDM_VARLIST *L)
+{
+  size_t i;
+  char str[128];
+  for (i= 0; i < L->nvars; i++)
+  {
+    UDM_VAR *V= &L->Var[i];
+    udm_snprintf(str, sizeof(str),
+                 "Section %s %d %d",
+                 V->name, V->section, V->maxlen);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmVarListPrint(FILE *f, UDM_VARLIST *L)
+{
+  size_t i;
+  char str[128];
+  for (i= 0; i < L->nvars; i++)
+  {
+    UDM_VAR *V= &L->Var[i];
+    if (!strcasecmp(V->name, "DBAddr")   ||
+        !strcasecmp(V->name, "ConfDir")  ||          /* TODO */
+        !strcasecmp(V->name, "ShareDir") ||          /* TODO */
+        !strcasecmp(V->name, "TmpDir")   ||          /* TODO */
+        !strcasecmp(V->name, "Request.User-Agent"))  /* TODO */
+      continue;
+    udm_snprintf(str, sizeof(str),
+                 "%s \"%s\"",
+                 V->name, V->val);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmDBListPrint(FILE *f, UDM_DBLIST *L)
+{
+  size_t i;
+  char str[128];
+  for (i= 0; i < L->nitems; i++)
+  {
+    const char *dbaddr;
+    dbaddr= UdmVarListFindStr(&L->db[i].Vars, "DBAddr", "<noaddr>");
+    udm_snprintf(str, sizeof(str), "DBAddr %s", dbaddr);
+    fprintf(f, "%s\n", str);
+  }                                
+  return UDM_OK;
+}
+
+
+static const char *
+UdmFollowTypeToStr(int follow)
+{
+  switch (follow)
+  {
+    case UDM_FOLLOW_NO:      return "page";
+    case UDM_FOLLOW_PATH:    return "path";
+    case UDM_FOLLOW_SITE:    return "site";
+    case UDM_FOLLOW_WORLD:   return "world";
+    case UDM_FOLLOW_URLLIST: return "urllist";
+    default: return "<UnknownFollowType>";
+  }
+}
+
+
+static const char *
+UdmMatchTypeToServerCommand(int match_type)
+{
+  switch (match_type)
+  {
+    case UDM_MATCH_WILD:   return "Realm";
+    case UDM_MATCH_REGEX:  return "Realm regex";
+    case UDM_MATCH_SUBNET: return "Subnet";
+    case UDM_MATCH_BEGIN:  return "Server";
+    default: return "<UnknownMatchType>";
+  }
+}
+
+
+static int
+UdmServerOptionsPrint(FILE *f, UDM_SERVER *Prev, UDM_SERVER *Curr)
+{
+  UDM_CONFCMD *cmd;
+  for (cmd= commands; cmd->name; cmd++)
+  {
+    if (cmd->action == srv_rpl_var      ||
+        cmd->action == srv_rpl_category ||
+        cmd->action == srv_rpl_num_var  ||
+        cmd->action == srv_rpl_bool_var ||
+        cmd->action == srv_rpl_time_var || 
+     /* cmd->action ==  srv_rpl_hdr     || */
+        cmd->action ==  srv_rpl_auth    ||
+     /* cmd->action ==  srv_rpl_mirror  || */
+        cmd->action ==  srv_rpl_charset ||
+        0)
+    {
+      const char *cval= UdmVarListFindStr(&Curr->Vars, cmd->name, "");
+      if (cmd->action == srv_rpl_auth)
+      {
+        if (cval[0])
+        {
+          size_t length;
+          char encoded[128], decoded[128];
+          udm_snprintf(encoded, sizeof(encoded), "%s", cval);
+          length= udm_base64_decode(decoded, encoded, sizeof(decoded));
+          fprintf(f, "%s '%s'\n", cmd->name, decoded);
+        }
+      }
+      else
+      {
+        const char *pval= Prev ? UdmVarListFindStr(&Prev->Vars, cmd->name, "") : "";
+        if (strcmp(pval, cval))
+          fprintf(f, "%s '%s'\n", cmd->name, cval);
+      }
+    }
+  }
+  return UDM_OK;
+}
+
+
+static size_t
+UdmServerToStr(char *str, size_t size, UDM_SERVER *S)
+{
+  const char *method= UdmVarListFindStr(&S->Vars, "Method", "Allow");
+  int follow= UdmVarListFindInt(&S->Vars, "Follow", UDM_FOLLOW_PATH);
+  int case_sense= UdmVarListFindBool(&S->Vars, "case_sense", UDM_CASE_INSENSITIVE);
+  int nomatch= UdmVarListFindBool(&S->Vars, "nomatch", 0);
+  const char *case_str= case_sense ? "" : "NoCase";
+  const char *match_str= nomatch ? "NoMatch" : "";
+  const char *follow_str= UdmFollowTypeToStr(follow);
+  const char *command= UdmMatchTypeToServerCommand(S->Match.match_type);
+  const char *alias= UdmVarListFindStr(&S->Vars, "Alias", "");
+  
+  /* TODO: Server site: cuts directory name */
+  switch(S->Match.match_type)
+  {
+    case UDM_MATCH_WILD:
+    case UDM_MATCH_REGEX:
+    case UDM_MATCH_SUBNET:
+      follow_str= "";
+    default: break;
+  }
+  
+  return udm_snprintf(str, size,
+                      "%s %s %s %s %s '%s'%s%s",
+                      command, follow_str, method,
+                      case_str, match_str, S->Match.pattern,
+                      alias[0] ? " " : "", alias);
+}
+
+
+static int
+UdmServerListPrint(FILE *f, UDM_SERVERLIST *L)
+{
+  size_t i;
+  char str[128];
+  for (i= 0; i < L->nservers; i++)
+  {
+    UDM_SERVER *S= &L->Server[i];
+    UDM_SERVER *P= i ? &L->Server[i-1] : NULL;
+/*    UdmVarListPrint(f, &S->Vars);*/
+    UdmServerOptionsPrint(f, P, S);
+    UdmServerToStr(str, sizeof(str), S);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmAliasListPrint(FILE *f, UDM_MATCHLIST *L, const char *cmd)
+{
+  size_t i;
+  char str[128];
+  for (i= 0; i < L->nmatches; i++)
+  {
+    UdmMatchToStr(str, sizeof(str), &L->Match[i], cmd);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmParserListPrint(FILE *f, UDM_PARSERLIST *L)
+{
+  size_t i;
+  char str[1024];
+  for (i= 0; i < L->nparsers; i++)
+  {
+    UDM_PARSER *P= &L->Parser[i];
+    udm_snprintf(str, sizeof(str),
+                 "Mime \"%s\" \"%s\" '%s'%s%s%s",
+                 P->from_mime, P->to_mime, P->cmd,
+                 P->src ? " \"" : "",
+                 P->src ? P->src : "",
+                 P->src ? "\"" : "");
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmAffixListListPrint(FILE *f, UDM_AFFIXLISTLIST *L)
+{
+  size_t i;
+  char str[256];
+  for (i= 0; i < L->nitems; i++)
+  {
+    UDM_AFFIXLIST *A= &L->Item[i];
+    udm_snprintf(str, sizeof(str),
+                 "Affix %s %s %s", A->lang, A->cset, A->fname);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmSpellListListPrint(FILE *f, UDM_SPELLLISTLIST *L)
+{
+  size_t i;
+  char str[256];
+  for (i= 0; i < L->nitems; i++)
+  {
+    UDM_SPELLLIST *S= &L->Item[i];
+    udm_snprintf(str, sizeof(str),
+                 "Spell %s %s %s", S->lang, S->cset, S->fname);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmLangmapListPrint(FILE *f, UDM_LANGMAPLIST *L)
+{
+  size_t i;
+  char str[256];
+  for (i= 0; i < L->nmaps; i++)
+  {
+    UDM_LANGMAP *M= &L->Map[i];
+    udm_snprintf(str, sizeof(str),
+                 "Langmap %s", M->filename);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmStopListListPrint(FILE *f, UDM_STOPLISTLIST *L)
+{
+  size_t i;
+  char str[256];
+  for (i= 0; i < L->nitems; i++)
+  {
+    UDM_STOPLIST *S= &L->Item[i];
+    udm_snprintf(str, sizeof(str),
+                 "StopwordFile %s", S->fname);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmSynonymListListPrint(FILE *f, UDM_SYNONYMLISTLIST *L)
+{
+  size_t i;
+  char str[256];
+  for (i= 0; i < L->nitems; i++)
+  {
+    UDM_SYNONYMLIST *S= &L->Item[i];
+    udm_snprintf(str, sizeof(str),
+                 "Synonym %s", S->fname);
+    fprintf(f, "%s\n", str);
+  }
+  return UDM_OK;
+}
+
+                    
+__C_LINK  int __UDMCALL UdmEnvSave(UDM_AGENT *Indexer,
+                                   const char *cname, int lflags)
+{
+  FILE *f;
+  UDM_ENV *E= Indexer->Conf;
+
+  if (!strcmp(cname, "-"))
+    f= stdout;
+  else if (!(f= fopen(cname,"w")))
+  {
+    sprintf(Indexer->Conf->errstr,
+            "Can't open output file '%s': %s", cname, strerror(errno));
+    return UDM_ERROR;
+  }
+
+  /* TODO: put interpreter line */
+  
+  UdmDBListPrint(f, &E->dbl);
+
+  fprintf(f, "LocalCharset %s\n", E->lcs->name);
+  fprintf(f, "BrowserCharset %s\n", E->bcs->name);
+
+  UdmSectionListPrint(f, &E->Sections);
+  UdmVarListPrint(f, &E->Vars);  
+
+  UdmMatchListPrint(f, &E->MimeTypes, "AddType");
+  UdmParserListPrint(f, &E->Parsers);
+  UdmMatchListPrint(f, &E->Filters, NULL);
+  
+  /*
+  UDM_MATCHLIST	SectionFilters;
+  UDM_MATCHLIST SectionHdrMatch;
+  UDM_MATCHLIST SectionGsrMatch;
+  UDM_MATCHLIST SectionMatch;
+  */
+
+  UdmStopListListPrint(f, &E->StopWord);
+  UdmSynonymListListPrint(f, &E->Synonym);
+  UdmAffixListListPrint(f, &E->Affixes);
+  UdmSpellListListPrint(f, &E->Spells);
+  UdmLangmapListPrint(f, &E->LangMaps);
+  
+  /*
+  int		CVS_ignore;
+  UDM_WORDPARAM	WordParam;
+  UDM_CHINALIST   Chi;
+  UDM_CHINALIST   Thai;
+#ifdef MECAB
+  mecab_t         *mecab;
+#endif
+  UDM_UNIDATA *unidata;
+  */
+
+
+  /*
+  UDM_VARLIST	XMLEnterHooks;
+  UDM_VARLIST	XMLLeaveHooks;
+  UDM_VARLIST	XMLDataHooks;
+  */
+
+  UdmAliasListPrint(f, &E->Aliases, "Alias");
+  UdmAliasListPrint(f, &E->ReverseAliases, "ReverseAlias");
+  UdmServerListPrint(f, &E->Servers);
+
+  if (f != stdout)
+    fclose(f);
+  return UDM_OK;
+}
