@@ -108,6 +108,25 @@ udmcrc32_t UdmCRC32Update(int prev,const char *b, size_t size)
 }
 
 
+udmcrc32_t UdmCRC32UpdateUnicode(int prev, const int *b, size_t charlen)
+{
+  udmcrc32_t crc= (udmcrc32_t)~prev;
+  const int *e;
+  
+  for (e= b + charlen; b < e; ++b)
+  {
+    char p1, p2;
+    p1= (char) (*b >> 8) & 0xFF;
+    p2= (char) (*b & 0xFF);
+    _CRC32_(crc, p1);
+    _CRC32_(crc, p2);
+  }
+  return (~crc);
+}
+
+
+
+
 #ifdef UDM_CRC32_TEST
 int main()
 {
