@@ -456,7 +456,7 @@ int UdmURLActionNoLock(UDM_AGENT *A, UDM_DOCUMENT *D, int cmd)
 
   for (i = dbfrom; i < dbto; i++)
   {
-    if (dbnum >= 0 && dbnum != i) continue;
+    if (dbnum >= 0 && dbnum != (int) i) continue;
     db = &A->Conf->dbl.db[i];
 
     UDM_GETLOCK(A, UDM_LOCK_DB);
@@ -841,7 +841,7 @@ UdmFindWordsMulDB(UDM_AGENT *A, UDM_RESULT *TmpRes,
     
   if (use_threads)
   {
-    unsigned long long ticks= UdmStartTimer();
+    udm_timer_t ticks= UdmStartTimer();
     UdmLog(A, UDM_LOG_DEBUG, "Creating threads");
     for (i= 0; i < ndatabases; i++)
     {
@@ -929,6 +929,99 @@ static int CreateAlias(UDM_ENV *Conf, UDM_DOCUMENT *Doc)
 }
 
 
+static int
+UdmAddCategoryInfo(UDM_AGENT *Agent)
+{
+  UDM_CATEGORY C;
+  UDM_ENV *Env= Agent->Conf;
+  size_t catcolumns= (size_t)atoi(UdmVarListFindStr(&Env->Vars,"CatColumns",""));
+
+  if (!catcolumns)
+    return UDM_OK;
+  
+  bzero((void*)&C, sizeof(C));
+  strcpy(C.addr,UdmVarListFindStr(&Env->Vars,"cat",""));
+
+  if(UDM_OK == UdmCatAction(Agent, &C, UDM_CAT_ACTION_LIST))
+  {
+    size_t n=1, l= 0, i;
+    char *catlist;
+    
+    for(i= 0; i < C.ncategories; i++)
+      l+= 128 + strlen(C.Category[i].path) + strlen(C.Category[i].name);
+      
+    if (l > 0  && (catlist= (char*)UdmMalloc(l)))
+    {
+      sprintf(catlist, "<table>\n");
+      for(i = 0; i < C.ncategories; i++)
+      {
+        sprintf(catlist+strlen(catlist),
+                "%s<td><a href=\"?cat=%s\">%s</A></td><td width=60>&nbsp;</td>\n",
+                n==1 ? "<tr>\n" : "",
+                C.Category[i].path, C.Category[i].name);
+      
+        if(n == catcolumns)
+        {
+          sprintf(catlist+strlen(catlist),"</tr>\n");
+          n=1;
+        }
+        else
+        {
+          n++;
+        }
+      }
+      sprintf(catlist + strlen(catlist), "</table>\n");
+      UdmVarListAddStr(&Env->Vars, "CS", catlist);
+      UDM_FREE(catlist);
+    }
+  }
+  else
+  {
+    return UDM_ERROR;
+  }
+    
+  UDM_FREE(C.Category);
+  bzero((void*)&C, sizeof(C));
+  strcpy(C.addr,UdmVarListFindStr(&Env->Vars,"cat",""));
+  
+  if(UDM_OK == UdmCatAction(Agent, &C, UDM_CAT_ACTION_PATH))
+  {
+    char *catpath = NULL;
+    size_t i, l= 0;
+    
+    for(i= 0; i < C.ncategories; i++)
+      l+= 32 + strlen(C.Category[i].path) + strlen(C.Category[i].name);
+    
+    if (l > 0 && (catpath= (char*)UdmMalloc(l)))
+    {
+      catpath[0]= '\0';
+      for(i = 0; i < C.ncategories; i++)
+      {
+        sprintf(catpath+strlen(catpath),"/<a href=\"?cat=%s\">%s</A>",
+                (C.Category[i].path) ? C.Category[i].path : "",
+                (C.Category[i].name) ? C.Category[i].name : "");
+      }
+      UdmVarListAddStr(&Env->Vars, "CP", catpath);
+      UDM_FREE(catpath);
+    }
+  }
+  else
+  {
+    return UDM_ERROR;
+  }
+  
+  UDM_FREE(C.Category);
+  return UDM_OK;
+}
+
+
+UDM_RESULT * __UDMCALL UdmFind2(UDM_AGENT *Agent, const char *query_string)
+{
+  UdmParseQueryString(Agent, &Agent->Conf->Vars, query_string);
+  return UdmFind(Agent);
+}
+
+
 UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
 {
   UDM_RESULT  *Res, *TmpRes;
@@ -940,6 +1033,7 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   size_t  offs=        (size_t) UdmVarListFindInt(&A->Conf->Vars, "offs", 0);
   size_t  ExcerptSize= (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptSize", 256);
   size_t  ExcerptPadding = (size_t)UdmVarListFindInt(&A->Conf->Vars, "ExcerptPadding", 40);
+  size_t  max_offs= 1000; /* Can be a search.htm command, make sure it is not overwritten by query string */
   char    str[128];
 
   UdmLog(A,UDM_LOG_DEBUG,"Start UdmFind");
@@ -950,13 +1044,19 @@ UDM_RESULT * __UDMCALL UdmFind(UDM_AGENT *A)
   TmpRes= (UDM_RESULT*) UdmMalloc(nbytes);
   bzero((void*) TmpRes, sizeof(UDM_RESULT) * numdatabases);
   Res=UdmResultInit(NULL);
-  if (UDM_OK != (res= UdmPrepare(A, Res)))
+  if (UDM_OK != (res= UdmPrepare(A, Res)) ||
+      UDM_OK != UdmAddCategoryInfo(A))
     goto conv;
   UdmVarListAddStr(&A->Conf->Vars, "orig_m", UdmVarListFindStr(&A->Conf->Vars, "m", "all"));
   ticks_= UdmStartTimer() - ticks_;
   UdmLog(A, UDM_LOG_DEBUG, "Stop  Prepare:\t\t%.2f", (float)ticks_/1000);
 
   Res->first = offs ? offs : page_number * page_size;  
+  if (Res->first + page_size > max_offs)
+  {
+    UdmLog(A, UDM_LOG_DEBUG, "Too large ps or np: offset=%d limit=%d\n", Res->first + page_size, max_offs);
+    goto conv;
+  }
 
   ticks_= UdmStartTimer();
   UdmLog(A, UDM_LOG_DEBUG, "Start FindWords");  
@@ -1202,7 +1302,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "oracle", UDM_DB_ORACLE8, UDM_DB_ORACLE8, 1, 
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
+    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT,
     &udm_sqldb_oracle_handler
   },
 #endif
@@ -1216,7 +1317,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "sybase", UDM_DB_SYBASE, UDM_DB_MSSQL, 1,
     UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX,
+    UDM_SQL_HAVE_TOP     | UDM_SQL_HAVE_0xHEX    |
+    UDM_SQL_HAVE_GOOD_COMMIT,
     &udm_sqldb_ctlib_handler
   },
 #endif
@@ -1259,7 +1361,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "sqlite3",
     UDM_DB_SQLITE3, UDM_DB_SQLITE3, 1,
-    UDM_SQL_HAVE_LIMIT | UDM_SQL_HAVE_GROUPBY | UDM_SQL_HAVE_STDHEX,
+    UDM_SQL_HAVE_LIMIT  | UDM_SQL_HAVE_GROUPBY |
+    UDM_SQL_HAVE_STDHEX | UDM_SQL_HAVE_GOOD_COMMIT, 
     &udm_sqldb_sqlite3_handler,
   },
 #endif
@@ -1303,7 +1406,8 @@ static UDM_SQLDB_DRIVER SQLDriver[]=
   {
     "odbc-oracle", UDM_DB_ORACLE8, UDM_DB_ODBC, 1,
     UDM_SQL_HAVE_GROUPBY   | UDM_SQL_HAVE_TRUNCATE |
-    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND | UDM_SQL_HAVE_ROWNUM,
+    UDM_SQL_HAVE_SUBSELECT | UDM_SQL_HAVE_BIND |
+    UDM_SQL_HAVE_ROWNUM    | UDM_SQL_HAVE_GOOD_COMMIT,
     &udm_sqldb_odbc_handler,
   },
   {
