@@ -76,7 +76,7 @@ __C_LINK void __UDMCALL UdmEnvFree(UDM_ENV * Env){
 	UdmHrefListFree(&Env->Hrefs);
 	UdmResultFree(&Env->Targets);
 	UdmParserListFree(&Env->Parsers);
-	UdmStopListFree(&Env->StopWords);
+	UdmStopListListFree(&Env->StopWord);
 	UdmHostListFree(&Env->Hosts);
 	
 	UdmMatchListFree(&Env->MimeTypes);
@@ -88,7 +88,7 @@ __C_LINK void __UDMCALL UdmEnvFree(UDM_ENV * Env){
 	UdmMatchListFree(&Env->SectionHdrMatch);
 	UdmMatchListFree(&Env->SectionGsrMatch);
 		
-	UdmSynonymListFree(&Env->Synonyms);
+	UdmSynonymListListFree(&Env->Synonym);
 	UdmVarListFree(&Env->Sections);
 	UdmVarListFree(&Env->XMLEnterHooks);
 	UdmVarListFree(&Env->XMLLeaveHooks);
@@ -154,6 +154,6 @@ int UdmEnvPrepare(UDM_ENV *Env)
         UdmAffixListListLoad(&Env->Affixes, flags, err, 128))
     return UDM_ERROR;
   }
-  UdmSynonymListSort(&Env->Synonyms);
+  UdmSynonymListListSortItems(&Env->Synonym);
   return UDM_OK;
 }
