@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -359,7 +359,7 @@ void UdmCheckLangMap(UDM_LANGMAP * map0, UDM_LANGMAP * map1, UDM_MAPSTAT *stat,
 
      stat->hits = stat->miss = 0;
      for (i = 0; i < UDM_LM_TOPCNT; i++) {
-       if ( (HIT = bsearch(&map1->memb[i], map0->memb, UDM_LM_TOPCNT, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpIndex)) == NULL) {
+       if ( (HIT = UdmBSearch(&map1->memb[i], map0->memb, UDM_LM_TOPCNT, sizeof(UDM_LANGITEM), (udm_qsort_cmp)UdmLMcmpIndex)) == NULL) {
 	 stat->miss++;
        } else {
 	 stat->hits += UDM_LM_TOPCNT - abs(HIT - map0->memb);
@@ -445,7 +445,8 @@ int  UdmGuessCharSet(UDM_AGENT *Indexer, UDM_DOCUMENT * Doc,UDM_LANGMAPLIST *Lis
           Choose charset in the same way.
      */
 
-     if (*charset == '\0' || *lang == '\0') {
+     if ((*charset == '\0' || *lang == '\0') && List->nmaps)
+     {
 
        /* Prepare document langmap */
        UdmPrepareLangMap(LangMap);
@@ -526,7 +527,7 @@ int  UdmGuessCharSet(UDM_AGENT *Indexer, UDM_DOCUMENT * Doc,UDM_LANGMAPLIST *Lis
      }
 
      if (have_server_lang && (*server_charset != '\0' || (*meta_charset != '\0' && use_meta ))) {
-       if (update_lm) {
+       if (update_lm && List->nmaps) {
          UDM_LANGMAP *Cmap;
 
 	 UDM_GETLOCK(Indexer,UDM_LOCK_CONF);
