@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 #include "udm_host.h"
 #include "udm_proto.h"
 #include "udm_utils.h"
+#include "udm_mutex.h"
 
 #ifndef INADDR_NONE
 #define INADDR_NONE ((unsigned long) -1)
@@ -104,10 +105,14 @@ static int host_addr_add(UDM_HOSTLIST *List,
   return 0;
 }
 
-static UDM_HOST_ADDR * host_addr_find(UDM_HOSTLIST *List,const char *hostname)
+UDM_HOST_ADDR *
+UdmHostFind(UDM_HOSTLIST *List,const char *hostname)
 {
   int l,c,r,res;
 
+  if (!hostname)
+    return NULL;
+  
   /* Find current URL in sorted part of list */
   l=0;r=List->nhost_addr-1;
   while(l<=r)
@@ -125,11 +130,31 @@ static UDM_HOST_ADDR * host_addr_find(UDM_HOSTLIST *List,const char *hostname)
 }
 
 
+void
+UdmHostErrorReset(UDM_HOSTLIST *List, const char *hostname)
+{
+  UDM_HOST_ADDR *h= UdmHostFind(List, hostname);
+  if (h)
+    h->net_errors= 0;
+}
+
+
+void
+UdmHostErrorIncrement(UDM_HOSTLIST *List, const char *hostname)
+{
+  UDM_HOST_ADDR *h= UdmHostFind(List, hostname);
+  if (h)
+    h->net_errors++;
+}
+
+
 int UdmHostLookup(UDM_HOSTLIST *List, UDM_CONN *connp)
 {
   struct hostent *he= NULL;
   UDM_HOST_ADDR  *Host;
 
+  connp->net_errors= 0;
+  
   if (!connp->hostname)
     return -1;
   
@@ -149,10 +174,10 @@ int UdmHostLookup(UDM_HOSTLIST *List, UDM_CONN *connp)
   {
     int i;
 
-    if((Host=host_addr_find(List,connp->hostname)))
+    if((Host= UdmHostFind(List,connp->hostname)))
     {
       Host->last_used=time(NULL);
-      connp->Host= Host;
+      connp->net_errors= Host->net_errors;
       if(Host->addr.s_addr)
       {
         memcpy(&connp->sin.sin_addr, &Host->addr, sizeof(struct in_addr));
@@ -187,15 +212,124 @@ int UdmHostLookup(UDM_HOSTLIST *List, UDM_CONN *connp)
   }
   else
   {
-    if(!(Host=host_addr_find(List,connp->hostname)))
+    if(!(Host= UdmHostFind(List,connp->hostname)))
     {
       host_addr_add(List,connp->hostname, &connp->sin.sin_addr);
     }
   }
-  connp->Host= host_addr_find(List, connp->hostname);
   return 0;
 }
 
+
+static int
+UdmHostLookupR(UDM_AGENT *A, UDM_HOSTLIST *List, UDM_CONN *connp)
+{
+#ifdef HAVE_GETHOSTBYNAME_R_6    
+  {
+    int err, rc;
+    char buf[2048];
+    struct hostent *he, res;
+    if (!(rc= gethostbyname_r(connp->hostname, &res,
+                              buf, sizeof(buf), &he, &err)) && he)
+    {
+      memcpy((char *)&(connp->sin.sin_addr), he->h_addr, (size_t)he->h_length);
+      return 0;
+    }
+    return -1;
+  }
+#else
+  {
+    int rc= -1;
+    struct hostent *he;
+    /*  gethostbyname() isn't reentrant */
+    UDM_GETLOCK(A, UDM_LOCK_CONF);
+    if ((he= gethostbyname(connp->hostname)))
+    {
+      memcpy((char *)&(connp->sin.sin_addr), he->h_addr, (size_t)he->h_length);
+      rc= 0;
+    }
+    UDM_RELEASELOCK(A, UDM_LOCK_CONF);
+    return rc;
+  }
+#endif
+  return -1;
+}
+
+
+int
+UdmHostLookup2(UDM_AGENT *A, UDM_HOSTLIST *List, UDM_CONN *connp)
+{
+  UDM_HOST_ADDR  *Host;
+
+  connp->net_errors= 0;
+  
+  if (!connp->hostname)
+    return -1;
+  
+  bzero((void*)&connp->sin, sizeof(struct sockaddr_in));
+  
+  /*  Set port  */
+  if (connp->port) 
+    connp->sin.sin_port= htons(connp->port);
+  else
+  {
+    connp->err= UDM_NET_ERROR;
+    return -1;
+  }
+
+  /* Check if hostname in standard numbers-and-dots notation */ 
+  if ((connp->sin.sin_addr.s_addr= inet_addr(connp->hostname)) == INADDR_NONE)
+  {
+    /* Not numbers-and-dots notation */
+    int rc= 0;
+
+    UDM_GETLOCK(A, UDM_LOCK_CONF);
+    if((Host= UdmHostFind(List,connp->hostname)))
+    {
+      Host->last_used=time(NULL);
+      connp->net_errors= Host->net_errors;
+      if(Host->addr.s_addr)
+      {
+        memcpy(&connp->sin.sin_addr, &Host->addr, sizeof(struct in_addr));
+      }
+      else
+      {
+        connp->err= UDM_NET_CANT_RESOLVE;
+        rc= -1;
+      }
+    }
+    UDM_RELEASELOCK(A, UDM_LOCK_CONF);
+    if (Host)
+      return rc;
+
+    rc= UdmHostLookupR(A, List, connp);
+    
+    if (rc < 0)
+    {
+      UDM_GETLOCK(A, UDM_LOCK_CONF);
+      host_addr_add(List,connp->hostname, NULL);
+      UDM_RELEASELOCK(A, UDM_LOCK_CONF);
+      connp->err= UDM_NET_CANT_RESOLVE;
+      return rc;
+    }
+    UDM_GETLOCK(A, UDM_LOCK_CONF);
+    host_addr_add(List,connp->hostname, &connp->sin.sin_addr);
+    UDM_RELEASELOCK(A, UDM_LOCK_CONF);
+  }
+  else
+  {
+    /* Numbers-and-dots notation */
+    UDM_GETLOCK(A, UDM_LOCK_CONF);
+    if(!(Host= UdmHostFind(List,connp->hostname)))
+    {
+      host_addr_add(List,connp->hostname, &connp->sin.sin_addr);
+    }
+    UDM_RELEASELOCK(A, UDM_LOCK_CONF);
+  }
+  return 0;
+}
+
+
 void UdmHostListFree(UDM_HOSTLIST *List)
 {
   size_t i;
