@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -88,6 +88,7 @@ __C_LINK int __UDMCALL UdmHrefListAdd(UDM_HREFLIST * HrefList,UDM_HREF * Href)
       HrefList->Href[c].server_id = Href->server_id;
       HrefList->Href[c].rec_id = Href->rec_id;
       HrefList->Href[c].max_doc_per_site = Href->max_doc_per_site;
+      HrefList->Href[c].collect_links= Href->collect_links;
       UdmVarListFree(&HrefList->Href[c].Vars);
       UdmVarListInit(&HrefList->Href[c].Vars);
       UdmVarListReplaceLst(&HrefList->Href[c].Vars, &Href->Vars, NULL, "*");
@@ -114,6 +115,7 @@ __C_LINK int __UDMCALL UdmHrefListAdd(UDM_HREFLIST * HrefList,UDM_HREF * Href)
       HrefList->Href[c].server_id = Href->server_id;
       HrefList->Href[c].rec_id = Href->rec_id;
       HrefList->Href[c].max_doc_per_site = Href->max_doc_per_site;
+      HrefList->Href[c].collect_links= Href->collect_links;
       UdmVarListFree(&HrefList->Href[c].Vars);
       UdmVarListInit(&HrefList->Href[c].Vars);
       UdmVarListReplaceLst(&HrefList->Href[c].Vars, &Href->Vars, NULL, "*");
@@ -135,6 +137,7 @@ __C_LINK int __UDMCALL UdmHrefListAdd(UDM_HREFLIST * HrefList,UDM_HREF * Href)
   HrefList->Href[HrefList->nhrefs].server_id = Href->server_id;
   HrefList->Href[HrefList->nhrefs].rec_id = Href->rec_id;
   HrefList->Href[HrefList->nhrefs].max_doc_per_site= Href->max_doc_per_site;
+  HrefList->Href[HrefList->nhrefs].collect_links= Href->collect_links;
   UdmVarListInit(&HrefList->Href[HrefList->nhrefs].Vars);
   UdmVarListReplaceLst(&HrefList->Href[HrefList->nhrefs].Vars,
                        &Href->Vars, NULL, "*");
