@@ -93,8 +93,6 @@
 
 /***************************************************************************/
 
-#define MAXHSIZE	1023*4	/* TUNE */
-
 static size_t UdmDocPerSiteCached(UDM_AGENT *Indexer,
                                   const char *hostinfo, size_t hostinfo_length)
 {
@@ -134,26 +132,24 @@ static int UdmCheckDocPerSite(UDM_AGENT *Indexer, UDM_HREF *Href, size_t *res,
 }
 
 
-__C_LINK int __UDMCALL UdmStoreHrefs(UDM_AGENT * Indexer)
+static int
+UdmHrefsCheckDocPerSite(UDM_AGENT *Indexer)
 {
   size_t i;
   int rc= UDM_OK;
-  UDM_DOCUMENT  Doc;
   UDM_HREFLIST *Hrefs;
   size_t doc_per_site= 0;
   size_t hostinfo_length= 0;
   char hostinfo[128]= "";
   
-  UdmDocInit(&Doc);
-
-  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+  UDM_LOCK_CHECK_OWNER(Indexer, UDM_LOCK_CONF);
   
   Hrefs= &Indexer->Conf->Hrefs;
-  
-  for (i = 0; i < Hrefs->nhrefs; i++)
+
+  for (i= 0; i < Hrefs->nhrefs; i++)
   {
     UDM_HREF *H = &Hrefs->Href[i];
-    if(H->stored == 0)
+    if (!H->stored)
     {
       if (H->max_doc_per_site)
       {
@@ -185,60 +181,35 @@ __C_LINK int __UDMCALL UdmStoreHrefs(UDM_AGENT * Indexer)
                  "Too many docs (%d) per site, skip it", doc_per_site);
           H->method = UDM_METHOD_DISALLOW;
           H->stored= 1;
-          continue;
         }
       }
-
-      UdmVarListAddLst(&Doc.Sections, &H->Vars, NULL, "*");
-      UdmVarListReplaceInt(&Doc.Sections, "Referrer-ID", H->referrer);
-      UdmVarListReplaceUnsigned(&Doc.Sections,"Hops", H->hops);
-      UdmVarListReplaceStr(&Doc.Sections,"URL",H->url?H->url:"");
-      UdmVarListReplaceInt(&Doc.Sections, "URL_ID", UdmStrHash32(H->url ? H->url : ""));
-      UdmVarListReplaceInt(&Doc.Sections,"Site_id", H->site_id);
-      UdmVarListReplaceInt(&Doc.Sections,"Server_id", H->server_id);
-      UdmVarListReplaceInt(&Doc.Sections, "HTDB_URL_ID", H->rec_id);
-      if (i >= Hrefs->dhrefs)
-      {
-        if(UDM_OK != (rc= UdmURLActionNoLock(Indexer, &Doc, UDM_URL_ACTION_ADD)))
-          goto ret;
-      }
-      if (H->collect_links)
-      {
-        if(UDM_OK != (rc= UdmURLActionNoLock(Indexer, &Doc, UDM_URL_ACTION_ADD_LINK)))
-          goto ret;
-      }
-      
-      UdmVarListFree(&Doc.Sections);
-      H->stored= 1;
     }
   }
-
 ret:
 
-  UdmDocFree(&Doc);
-  
-  /*
-    Remember last stored URL num
-    Note that it will became 0
-    after next sort in AddUrl   
-  */
-  Hrefs->dhrefs = Hrefs->nhrefs;
-  
-  /*
-    We should not free URL list with onw database
-    to avoid double indexing of the same document
-    So, do it if compiled with SQL only          
-  */
-  
-  /* FIXME: this is incorrect with both SQL and built-in compiled */
-  if(Hrefs->nhrefs > MAXHSIZE)
-    UdmHrefListFree(&Indexer->Conf->Hrefs);
-
-  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
-
   return rc;
 }
 
+
+__C_LINK int __UDMCALL
+UdmStoreHrefs(UDM_AGENT *Indexer)
+{
+  int rc;
+  
+  UDM_GETLOCK(Indexer, UDM_LOCK_CONF);
+
+  if (UDM_OK != (rc= UdmHrefsCheckDocPerSite(Indexer)))
+    goto ret; 
+  
+  rc= UdmStoreHrefsSQL(Indexer);
+
+ret:
+
+  UDM_RELEASELOCK(Indexer, UDM_LOCK_CONF);
+  return rc;
+}
+
+
 static void RelLink(UDM_URL *curURL, UDM_URL *newURL, char **str)
 {
   const char  *schema = newURL->schema ? newURL->schema : curURL->schema;
@@ -341,7 +312,10 @@ UdmFilterFind(UDM_MATCHLIST *L,const char *newhref,char *reason)
 #endif
   if ((M= UdmMatchListFind(L, newhref, 0, NULL)))
   {
-    sprintf(reason,"%s %s %s '%s'",M->arg,UdmMatchTypeStr(M->match_type),M->case_sense?"Sensitive":"InSensitive",M->pattern);
+    sprintf(reason,"%s %s %s '%s'",
+            M->arg, UdmMatchTypeStr(M->match_type),
+            M->case_sense == UDM_CASE_INSENSITIVE ? "InSensitive":"Sensitive",
+            M->pattern);
     res=UdmMethod(M->arg);
   }
   else
@@ -401,7 +375,7 @@ UdmSectionFilterFind(UDM_MATCHLIST *L, UDM_DOCUMENT *Doc, char *reason)
     sprintf(reason,"%s %s %s %s '%s' '%s'", M->arg,
             M->nomatch ? "NoMatch" : "Match",
             UdmMatchTypeStr(M->match_type),
-            M->case_sense ? "Sensitive" : "InSensitive",
+            M->case_sense == UDM_CASE_INSENSITIVE ? "InSensitive" : "Sensitive",
             M->section,
             M->pattern);
     res= UdmMethod(M->arg);
@@ -1320,24 +1294,25 @@ UdmDocContentDecode(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
 {
   const char  *url=UdmVarListFindStr(&Doc->Sections,"URL","");
   const char  *ce=UdmVarListFindStr(&Doc->Sections,"Content-Encoding","");
+  int code;
 
 #ifdef HAVE_ZLIB
   if(!strcasecmp(ce,"gzip") || !strcasecmp(ce,"x-gzip"))
   {
     UDM_THREADINFO(Indexer,"UnGzip",url);
-    UdmUnGzip(Doc);
+    code= UdmUnGzip(Doc);
     UdmVarListReplaceInt(&Doc->Sections, "Content-Length", Doc->Buf.buf - Doc->Buf.content + (int)Doc->Buf.size);
   }
   else if(!strcasecmp(ce,"deflate"))
   {
     UDM_THREADINFO(Indexer,"Inflate",url);
-    UdmInflate(Doc);
+    code= UdmInflate(Doc);
     UdmVarListReplaceInt(&Doc->Sections, "Content-Length", Doc->Buf.buf - Doc->Buf.content + (int)Doc->Buf.size);
   }
   else if(!strcasecmp(ce,"compress") || !strcasecmp(ce,"x-compress"))
   {
     UDM_THREADINFO(Indexer,"Uncompress",url);
-    UdmUncompress(Doc);
+    code= UdmUncompress(Doc);
     UdmVarListReplaceInt(&Doc->Sections, "Content-Length", Doc->Buf.buf - Doc->Buf.content + (int)Doc->Buf.size);
   }
   else
@@ -1351,6 +1326,16 @@ UdmDocContentDecode(UDM_AGENT *Indexer, UDM_DOCUMENT *Doc)
     UdmLog(Indexer,UDM_LOG_ERROR,"Unsupported Content-Encoding");
     UdmVarListReplaceInt(&Doc->Sections,"Status",UDM_HTTP_STATUS_UNSUPPORTED_MEDIA_TYPE);
   }
+  
+  if (code == -1)
+  {
+    const char  *te= UdmVarListFindStr(&Doc->Sections,"Transfer-Encoding", NULL);
+    if (te)
+      UdmLog(Indexer, UDM_LOG_ERROR, "Transfer-Encoding '%s' is not supported", te);
+    UdmLog(Indexer, UDM_LOG_ERROR, "Content-Encoding processing failed");
+    Doc->Buf.content[0]= '\0';
+    Doc->Buf.size= Doc->Buf.content - Doc->Buf.buf;
+  }
   return UDM_OK;
 }
 
@@ -2286,7 +2271,7 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
         
         if(wlen>Indexer->Conf->WordParam.max_word_len ||
            wlen<Indexer->Conf->WordParam.min_word_len ||
-           UdmStopListFind(&Indexer->Conf->StopWords, w, lang ) != NULL)
+           UdmStopListListFind(&Indexer->Conf->StopWord, w, lang ) != NULL)
         {
           Doc.Words.Word[wordnum].secno= 0;
         }  
@@ -2298,7 +2283,7 @@ __C_LINK int __UDMCALL UdmIndexNextURL(UDM_AGENT *Indexer){
         
         if(wlen>Indexer->Conf->WordParam.max_word_len ||
            wlen<Indexer->Conf->WordParam.min_word_len ||
-           UdmStopListFind(&Indexer->Conf->StopWords,w, lang) != NULL)
+           UdmStopListListFind(&Indexer->Conf->StopWord, w, lang) != NULL)
         {
           Doc.CrossWords.CrossWord[wordnum].weight=0;
         }  
