@@ -172,14 +172,14 @@ static int udm_logger(UDM_ENV *Env, int handle, int level, const char *fmt, va_l
 	char buf[UDM_LOG_BUF_LEN+1];
 	int i=0;
 
+	if (handle)i=snprintf(buf,UDM_LOG_BUF_LEN,"[%d]{%02d} ", (int)getpid(), handle);
 #ifdef WIN32
-	_vsnprintf(buf,UDM_LOG_BUF_LEN,fmt,ap);
+	_vsnprintf(buf+i,UDM_LOG_BUF_LEN,fmt,ap);
 	if(Env->ThreadInfo)
 		Env->ThreadInfo(0,"",buf);
 	else
 		printf("%s\n",buf);
 #else
-	if (handle)i=snprintf(buf,UDM_LOG_BUF_LEN,"[%d]{%02d} ", (int)getpid(), handle);
 	vsnprintf(buf+i,(size_t)(UDM_LOG_BUF_LEN-i),fmt,ap);
 #endif
 
