@@ -75,6 +75,7 @@ static int add_servers=UDM_FLAG_ADD_SERV;
 static int add_server_urls = UDM_FLAG_ADD_SERVURL;
 static int load_langmaps=UDM_FLAG_LOAD_LANGMAP;
 static int load_spells=UDM_FLAG_SPELL;
+static int load_for_dump= 0;
 static int warnings=1;
        int maxthreads=1;
        int indexing = 0;
@@ -260,6 +261,7 @@ static int ShowStatistics(UDM_AGENT *Indexer)
   stat_time = UdmVarListFindStr(&Conf.Vars, "stat_time", "0");
   bzero(&tm, sizeof(tm));
 
+#ifndef WIN32
   if (stat_time &&
       strlen(stat_time) >= 7 &&
       stat_time[4] == '-' &&
@@ -283,7 +285,13 @@ static int ShowStatistics(UDM_AGENT *Indexer)
     Stats.time = time(NULL);
     localtime_r(&Stats.time, &tm);
   }
-
+#else
+  {
+	struct tm *tm1;
+    Stats.time= time(NULL);
+    tm1= localtime(&Stats.time);
+  }
+#endif
   strftime(sbuf, sizeof(sbuf), "%Y-%m-%d %H:%M:%S", &tm);
   res=UdmStatAction(Indexer,&Stats);
 
@@ -679,6 +687,7 @@ static struct udm_indcmd2str_st indcmd[]=
   {"rewritelimits", UDM_IND_REWRITELIMITS},
   {"hashspell",     UDM_IND_HASHSPELL},
   {"dumpspell",     UDM_IND_DUMPSPELL},
+  {"dumpconf",      UDM_IND_DUMPCONF},
   {"",              UDM_IND_INDEX}
 };
 
@@ -841,10 +850,10 @@ httpd_client_handler(int client, UDM_AGENT *A)
   char speed_info[128]= "";
   ssize_t nrecv,nsent;
   size_t i, len, total_docs= 0, total_sec= 0;
-  unsigned long long total_bytes= 0;
+  udm_uint8 total_bytes= 0;
   time_t now= time(0);
   
-  nrecv= recv(client, &request, sizeof(request), 0);
+  nrecv= recv(client, request, sizeof(request), 0);
   UdmLog(A, UDM_LOG_ERROR, "Received request len=%d", (int) nrecv);
   udm_snprintf(response, sizeof(response) - 1,
                "HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n");
@@ -938,7 +947,9 @@ static void* thread_main_httpd(void *arg)
 #endif
 {
   UDM_AGENT *A= (UDM_AGENT*) arg;
+#ifndef WIN32
   UdmStartHTTPD(A, httpd_client_handler);
+#endif
   return 0;
 }
 
@@ -957,7 +968,7 @@ static void * thread_main(void *arg)
   int res=UDM_OK;
   int done=0;
   int i_sleep=0;
-     
+ 
   while (!done)
   {
     if (max_index_time>=0)
@@ -1123,7 +1134,6 @@ static void * thread_main(void *arg)
   {
     time_t now, sec;
     float M = 0.0, K = 0.0;
-    int z;
 
     UdmURLAction(Indexer, NULL, UDM_URL_ACTION_FLUSH); /* flush DocCache */
     UdmWordCacheFlush(Indexer);
@@ -1132,23 +1142,27 @@ static void * thread_main(void *arg)
     sec= now - Indexer->start_time - Indexer->nsleepsecs;
     if (sec > 0)
     {
-      M= Indexer->nbytes / 1048576.0 / sec;
-      if (M < 1.0) K = Indexer->nbytes / 1024.0 / sec;
+      /* Convert to int64 - conversion from uint64 to double doesn't work on windows */
+      M= ((udm_timer_t)Indexer->nbytes) / 1048576.0 / sec;
+      if (M < 1.0) K = ((udm_timer_t)Indexer->nbytes) / 1024.0 / sec;
     }
     UdmLog(Indexer,UDM_LOG_ERROR,"Done (%d seconds, %u documents, %llu bytes, %5.2f %cbytes/sec.)",
            sec, Indexer->ndocs, Indexer->nbytes, (M < 1.0) ? K : M, (M < 1.0) ? 'K' : 'M' );
 #if !defined(WIN32) && defined(HAVE_PTHREAD)
-    for (z = 0 ; z < maxthreads; z++)
+	{
+      int z;
+      for (z = 0 ; z < maxthreads; z++)
       if (ThreadIndexers[z].handle)
         pthread_kill(threads[z], SIGALRM); /* wake-up sleeping threads */
+	}
 #endif
   }
   UDM_GETLOCK(Indexer, UDM_LOCK_THREAD);
   total_threads--;
-  UdmAgentFree(Indexer);
-  Indexer->handle= 0;
   UDM_RELEASELOCK(Indexer, UDM_LOCK_THREAD);
 
+  UdmAgentFree(Indexer);
+
 #ifdef WIN32
   return(0);
 #else
@@ -1249,7 +1263,11 @@ static int UdmClear(UDM_AGENT *A, const char *url_fname)
 
 
 static int
+#ifdef WIN32
+UdmThreadCreate(udm_thread_t *thd, unsigned (__stdcall *start_routine)(void *), void *arg)
+#else
 UdmThreadCreate(udm_thread_t *thd, void *(*start_routine)(void*), void *arg)
+#endif
 {
   int res;
 #ifdef WIN32
@@ -1483,7 +1501,11 @@ UdmEnvSetDirs(UDM_ENV *Env)
   {
     char *conf_dir= WinUdmConfDir();
     UdmVarListReplaceStr(&Env->Vars,"ConfDir", conf_dir);
-    UdmVarListReplaceStr(&Env->Vars,"ShareDir", conf_dir);
+	{
+	  char dir[256];
+	  udm_snprintf(dir, sizeof(dir), "%s\\%s", conf_dir, "create");
+      UdmVarListReplaceStr(&Env->Vars,"ShareDir", dir);
+	}
     UdmVarListReplaceStr(&Env->Vars,"TmpDir", conf_dir);
     UdmVarListReplaceStr(&Env->Vars,"VarDir", conf_dir);
     UDM_FREE(conf_dir);
@@ -1555,15 +1577,21 @@ int main(int argc, char **argv)
     fprintf(stderr, "Unknown indexer command in -E\n");
     help++;
   }
-  
-  if (cmd != UDM_IND_INDEX)
+
+  if (cmd == UDM_IND_DUMPCONF)
+  {
+    load_for_dump|= UDM_FLAG_DONT_ADD_TO_DB;
+    load_langmaps= 0;
+    load_spells= 0;
+  }
+  else if (cmd != UDM_IND_INDEX)
   {
     add_servers=0;
     load_langmaps=0;
     if (cmd != UDM_IND_HASHSPELL && cmd  != UDM_IND_DUMPSPELL)
       load_spells=0;
   }
-     
+
   flags|=add_servers;
   flags |= add_server_urls;
   Main.flags = flags;
@@ -1593,12 +1621,23 @@ int main(int argc, char **argv)
     }
   }
      
-  if (UDM_OK != UdmIndexerEnvLoad(&Main, cname, add_servers + load_langmaps + load_spells + add_server_urls))
+  if (UDM_OK != UdmIndexerEnvLoad(&Main, cname,
+                                  add_servers + load_langmaps + load_spells +
+                                  add_server_urls + load_for_dump))
   {
     fprintf(logfile, "%s\n", UdmEnvErrMsg(&Conf));
     UdmEnvFree(&Conf);
     return 1;
   }
+
+ 
+  if (cmd == UDM_IND_DUMPCONF)
+  {
+    if (UDM_OK != (rc= UdmEnvSave(&Main, "-", 0)))
+      fprintf(logfile, "%s\n", UdmEnvErrMsg(&Conf));
+    UdmEnvFree(&Conf);
+    return rc;
+  }
      
   if (cmd == UDM_IND_CHECKCONF)
     return 0;
@@ -1607,7 +1646,6 @@ int main(int argc, char **argv)
    UdmSetLogLevel(NULL, have_loglevel ? loglevel :
                   UdmVarListFindInt(&Main.Conf->Vars, "LogLevel", UDM_LOG_INFO));
    UdmOpenLog("indexer",&Conf, log2stderr);
-   UdmSigHandlersInit(&Main);
 #endif
    
   if (REQUEST_METHOD)
@@ -1800,6 +1838,7 @@ int main(int argc, char **argv)
         }
         UdmLog(&Main,UDM_LOG_ERROR, "indexer from %s-%s-%s started with '%s'", PACKAGE, VERSION, UDM_DBTYPE, cname);
         UdmStoreHrefs(&Main);    /**< store hrefs from config and command line */
+        UdmSigHandlersInit(&Main);
         rc= UdmIndex(&Main);
       }
   }
