@@ -57,7 +57,7 @@ int UdmMatchComp(UDM_MATCH *Match,char *errstr,size_t errstrsize)
     case UDM_MATCH_REGEX:
       Match->reg = (regex_t*)UdmRealloc(Match->reg, sizeof(regex_t));
       bzero((void*)Match->reg, sizeof(regex_t));
-      if(Match->case_sense)
+      if(Match->case_sense == UDM_CASE_INSENSITIVE)
         flag|=REG_ICASE;
       if((err=regcomp(Match->reg, Match->pattern, flag)))
       {
@@ -67,7 +67,8 @@ int UdmMatchComp(UDM_MATCH *Match,char *errstr,size_t errstrsize)
       }
       break;
     case UDM_MATCH_WILD:
-      if (IsSimpleWildEnding(Match->pattern))
+      if (!(Match->flags & UDM_MATCH_FLAG_SKIP_OPTIMIZATION) &&
+          IsSimpleWildEnding(Match->pattern))
       {
         /* Convert simple UDM_MATCH_WILD cases to UDM_MATCH_END */
         memmove(Match->pattern, Match->pattern + 1, Match->pattern_length);
@@ -146,7 +147,7 @@ int UdmMatchExec(UDM_MATCH * Match,
       break;
     case UDM_MATCH_WILD:
       for(i=0;i<nparts;i++)Parts[i].beg=Parts[i].end=-1;
-      if(Match->case_sense)
+      if(Match->case_sense == UDM_CASE_INSENSITIVE)
       {
         res=UdmWildCaseCmp(string,Match->pattern);
       }
@@ -157,7 +158,7 @@ int UdmMatchExec(UDM_MATCH * Match,
       break;
     case UDM_MATCH_SUBNET:
       for(i = 0; i < nparts; i++) Parts[i].beg = Parts[i].end = -1;
-      if(Match->case_sense)
+      if(Match->case_sense == UDM_CASE_INSENSITIVE)
       {
         res = UdmWildCaseCmp(net_string, Match->pattern);
       }
@@ -168,7 +169,7 @@ int UdmMatchExec(UDM_MATCH * Match,
       break;
     case UDM_MATCH_BEGIN:
       for(i=0;i<nparts;i++)Parts[i].beg=Parts[i].end=-1;
-      if(Match->case_sense)
+      if(Match->case_sense == UDM_CASE_INSENSITIVE)
       {
         res=strncasecmp(Match->pattern, string, Match->pattern_length);
       }
@@ -179,7 +180,7 @@ int UdmMatchExec(UDM_MATCH * Match,
       break;
     case UDM_MATCH_FULL:
       for(i=0;i<nparts;i++)Parts[i].beg=Parts[i].end=-1;
-      if(Match->case_sense)
+      if(Match->case_sense == UDM_CASE_INSENSITIVE)
       {
         res=strcasecmp(Match->pattern,string);
       }
@@ -196,7 +197,7 @@ int UdmMatchExec(UDM_MATCH * Match,
         break;
       }
       se= string + string_length - Match->pattern_length;
-      if(Match->case_sense)
+      if(Match->case_sense == UDM_CASE_INSENSITIVE)
       {
         res=strcasecmp(Match->pattern,se);
       }
