@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #endif
 
 #include "udm_uniconv.h"
+#include "udm_utils.h"
 
 #define MCONV_BUF_SIZE 4096
 
@@ -131,11 +132,13 @@ Please post bug reports and suggestions at http://www.mnogosearch.org/bugs/\n",
 int main(int argc, char **argv)
 {
   char *charset_from = NULL, *charset_to = NULL;
-  UDM_CHARSET *CH_F, *CH_T, *sys_int;
-  UDM_CONV fc_uni, uni_tc;
-  int html_from = 0, html_to = 0;
-  char *from_buf, *uni_buf, *to_buf;
-  int ch, help = 0, verbose = 0;
+  UDM_CHARSET *CH_F, *CH_T;
+  UDM_CONV conv;
+  int html_from= 0, html_to= 0;
+  int ch, help= 0, verbose= 0;
+  char from_buf[1024];
+  char to_buf[1024*8];
+  size_t nbytes;
 
   while ((ch = getopt(argc, argv, "Eehv?t:f:")) != -1)
   {
@@ -180,36 +183,15 @@ int main(int argc, char **argv)
     }
     exit(1);
   }
-  sys_int= &udm_charset_sys_int;
 
-  UdmConvInit(&fc_uni, CH_F, sys_int, (html_from) ? UDM_RECODE_HTML : 0);
-  UdmConvInit(&uni_tc, sys_int, CH_T, (html_to) ? UDM_RECODE_HTML : 0);
+  UdmConvInit(&conv, CH_F, CH_T, (html_from) ? UDM_RECODE_HTML : 0);
 
-  if ((from_buf = (char*)UdmMalloc(MCONV_BUF_SIZE)) == NULL)
+  while((nbytes= fread(from_buf, 1, sizeof(from_buf), stdin)))
   {
-    if (verbose) fprintf(stderr, "Can't alloc memory, %d bytes", MCONV_BUF_SIZE);
-    exit(1);
-  }
-  if ((to_buf = (char*)UdmMalloc(16 * MCONV_BUF_SIZE)) == NULL)
-  {
-    if (verbose) fprintf(stderr, "Can't alloc memory, %d bytes", 16 * MCONV_BUF_SIZE);
-    exit(1);
-  }
-  if ((uni_buf = (char*)UdmMalloc(4 * MCONV_BUF_SIZE * sizeof(int))) == NULL)
-  {
-    if (verbose) fprintf(stderr, "Can't alloc memory, %d bytes", 4 * MCONV_BUF_SIZE * sizeof(int));
-    exit(1);
-  }
-
-  while((fgets(from_buf, MCONV_BUF_SIZE, stdin)) != NULL)
-  {
-    UdmConv(&fc_uni, uni_buf, 4 * MCONV_BUF_SIZE * sizeof(int), from_buf, MCONV_BUF_SIZE);
-    UdmConv(&uni_tc, to_buf, 16 * MCONV_BUF_SIZE, uni_buf, 4 * MCONV_BUF_SIZE * sizeof(int));
-    if (fputs(to_buf, stdout) < 0)
-    {
-      if (verbose) fprintf(stderr, "An output error ocurred");
-      exit(1);
-    }
+    nbytes= UdmConv(&conv, to_buf, sizeof(to_buf), from_buf, nbytes);
+    nbytes= fwrite(to_buf, 1, nbytes, stdout);
+    if (feof(stdin))
+      break;
   }
 
   if (ferror(stdin))
