@@ -207,12 +207,17 @@ int UdmPrepareWords(UDM_AGENT * Indexer,UDM_DOCUMENT * Doc){
     and so on to improve clone detection quality
     */
     
-    if (!Sec || !(Sec->flags & UDM_VARFLAG_NOCLONE))
+    if (Doc->Spider.index && (!Sec || !(Sec->flags & UDM_VARFLAG_NOCLONE)))
     {
       crc32= UdmCRC32UpdateUnicode(crc32, ustr, ustr_char_len);
     }
     
-    if(Item->section)
+    /*
+      We need to check Doc->Spider.index, because
+      user-defined sections must respect 
+      <META NAME="Robots" CONTENT="NOINDEX">
+    */
+    if(Doc->Spider.index && Item->section)
     {
       if (strip_accents)
         UdmUniStrStripAccents(unidata, ustr);
