@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -323,10 +323,11 @@ static char *parse_file(UDM_AGENT *Agent,
   char *result=NULL;
   char *arg1pos,*arg2pos;
   int parser_type;
-  char fn0[1024]="/tmp/" "indXXXXXX";
+  char fn0[1024]="";
   char fn1[1024]="";
   char * fnames[2];
   const char *url= UdmVarListFindStr(&Doc->Sections,"URL","");
+  const char *tmpdir= UdmVarListFindStr(&Agent->Conf->Vars, "TmpDir", UDM_TMP_DIR);
 
   arg1pos=strstr(parser->cmd,"$1");
   arg2pos=strstr(parser->cmd,"$2");
@@ -341,7 +342,7 @@ static char *parse_file(UDM_AGENT *Agent,
   */
   sprintf(fn0, ".\\prstmp%d",Agent->handle);
 #else
-  sprintf(fn0,"/tmp/ind.%d.%d", Agent->handle, getpid());
+  sprintf(fn0,"%s/ind.%d.%d", tmpdir, Agent->handle, getpid());
 #endif
   strcpy(fn1,fn0);
   fnames[0]=strcat(fn0,".in");
@@ -369,7 +370,7 @@ static char *parse_file(UDM_AGENT *Agent,
 #else
     umask((mode_t)022);
 #endif
-    fd=open(fnames[0],O_RDWR|O_CREAT|UDM_BINARY,UDM_IWRITE);
+    fd=open(fnames[0],O_RDWR|O_CREAT|O_TRUNC|UDM_BINARY,UDM_IWRITE);
     if (fd < 0)
     {
       UdmLog(Agent,UDM_LOG_ERROR,"Can't open output file '%s'", fnames[0]);
