@@ -197,7 +197,7 @@ int __UDMCALL UdmFindWordsSearchd(UDM_AGENT *A,UDM_RESULT *Res, UDM_DB *db)
   int port= UdmVarListFindInt(&db->Vars, "DBPort", 80);
   int lookup_error, rc;
   size_t length;
-  unsigned long long ticks;
+  udm_timer_t ticks;
 
   UdmDocInit(&Inc);
   Inc.Buf.maxsize= UDM_MAXDOCSIZE;
