@@ -32,6 +32,10 @@
 #include <locale.h>
 #endif
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 #include "udmsearch.h"
 #include "udm_xmalloc.h"
 
@@ -46,6 +50,10 @@
 
 #define POWERED_LOGO "<BR><a href=\"http://www.mnogosearch.org/\"><IMG BORDER=0 SRC=\"http://www.mnogosearch.org/img/mwin.gif\"></A>&nbsp<FONT SIZE=1><A HREF=\"http://www.mnogosearch.org/\">Powered by mnoGoSearch - free web search engine software</A></FONT><BR>\n"
 
+#define UDM_HTTPD_CGI    1
+#define UDM_HTTPD_DAEMON 2
+
+
 /******************** Misc functions *********************************/
 
 static void ParseQStringUnescaped(UDM_VARLIST *vars, const char *qstring)
@@ -147,7 +155,8 @@ static int UdmLoadGroups (UDM_ENV *E) {
 #endif
 
 /************************************************************/
-static int StoreDocCGI (UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl)
+static int
+StoreDocCGI(UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl, int httpd)
 {
   UDM_DOCUMENT *Doc;
   UDM_RESULT   *Res;
@@ -189,6 +198,8 @@ static int StoreDocCGI (UDM_AGENT *Agent, UDM_ENV *Env, UDM_VARLIST *tmpl)
   charset = UdmVarListFindStr(&Env->Vars,"Charset","iso-8859-1");
   cs = UdmGetCharSet(charset);
   
+  if (httpd & UDM_HTTPD_DAEMON)
+    printf("HTTP/1.0 200 OK\r\n");
   printf("Content-type: text/html; charset=%s\r\n\r\n", charset);
   
   /* Now start displaying template*/
@@ -312,10 +323,12 @@ int UdmThreadJoin(void *thd)
 }        
 #endif
 
-
 static int
 UdmHTTPHeadersSend(int httpd, const char *content_type, const char *bcs)
 {
+  if (httpd & UDM_HTTPD_DAEMON)
+    printf("HTTP/1.0 200 OK\r\n");
+  
   if (httpd && strcasecmp(content_type, "none"))
   {
     if (bcs)
@@ -327,15 +340,162 @@ UdmHTTPHeadersSend(int httpd, const char *content_type, const char *bcs)
 }
 
 
+static void usage(void)
+{
+}
+
+
+typedef struct udm_self_template_query_st
+{
+  char template_name[1024];
+  char self[1024];
+  char query_string[1024];
+} UDM_SELF_TEMPLATE_QUERY;
+
+
+static int
+UdmDetectSelfTemplateQueryCGI(UDM_ENV *Env,
+                              UDM_SELF_TEMPLATE_QUERY *q,
+                              int argc, char **argv)
+{
+  char *env;
+  /* Detect self */
+  if((env= getenv("UDMSEARCH_SELF")) || (env= getenv("REQUEST_URI")))
+  {
+    /* Use Apache's REQUEST_URI, which is not in CGI standard */
+    char *end;
+    udm_snprintf(q->self, sizeof(q->self), "%s", env);
+    if ((end= strchr(q->self,'?')))
+      end[0]='\0';
+  }
+  else if ((env= getenv("SCRIPT_NAME")))
+  {
+    /* Use standard CGI variables if not under Apache */
+    const char *path_info= getenv("PATH_INFO");
+    udm_snprintf(q->self, sizeof(q->self),"%s%s", env, path_info ? path_info:"");
+  }
+  else
+  {
+    udm_snprintf(q->self, sizeof(q->self), "search.cgi");
+  }
+  
+  
+  /* Detect template name */
+  if((env= getenv("UDMSEARCH_TEMPLATE")))
+  {
+    char ename[1024];
+    strncpy(ename,env,sizeof(ename)-1);
+    ename[sizeof(ename)]='\0';
+    UdmUnescapeCGIQuery(q->template_name, ename);
+  }
+  else if(getenv("REDIRECT_STATUS") && (env= getenv("PATH_TRANSLATED")))
+  {
+    /* Check Apache internal redirect  */
+    /* via   "AddHandler" and "Action" */
+    udm_snprintf(q->template_name, sizeof(q->template_name), "%s", env);
+  }
+  else if ((env= getenv("PATH_INFO")) && env[0])
+  {
+    /* http://localhost/cgi-bin/search.cgi/path/to/search.htm */
+    char name[128], *conf_dir;
+    
+    strncpy(name,env,sizeof(name)-1);
+    name[sizeof(name)-1]='\0';
+    if (UDMSLASH !='/')
+    {
+      char *s;
+      for (s=name; s[0]; s++)
+        if (s[0]=='\\')s[0]='/';
+    }
+    
+    conf_dir= ((env= getenv("UDM_CONF_DIR")) || (env= getenv("UDM_ETC_DIR"))) ?
+               (char *)UdmStrdup(env) : UdmConfDir();
+    /* Take from the config directory */
+    udm_snprintf(q->template_name, sizeof(q->template_name)-5, "%s%s%s", 
+                 conf_dir,UDMSLASHSTR,name);
+    UDM_FREE(conf_dir);
+  }
+  else
+  {
+    /* CGI executed without Apache internal redirect */
+    /* or started from command line                  */
+    char *end, *conf_dir;
+    size_t length;
+    
+    conf_dir= ((env= getenv("UDM_CONF_DIR")) || (env= getenv("UDM_ETC_DIR"))) ?
+               (char *)UdmStrdup(env) : UdmConfDir();
+    
+    /* Take from the config directory */
+    length= udm_snprintf(q->template_name, sizeof(q->template_name)-5, "%s%s%s", 
+                         conf_dir,UDMSLASHSTR,
+                         (end= strrchr(q->self,'/')) ? (end+1) : (q->self));
+    end= q->template_name + length;
+    if ( length > 3 && (!strcmp(end-4,".exe") || !strcmp(end-4,".cgi")))
+      end-=4;
+    strcpy(end,".htm");
+    UDM_FREE(conf_dir);
+  }
+  
+  if((env= getenv("QUERY_STRING")))
+  {
+    udm_snprintf(q->query_string, sizeof(q->query_string), "%s", env);
+  }
+  else if(argv[0])
+  {
+    udm_snprintf(q->query_string, sizeof(q->query_string), "q=%s", argv[0]);
+    UdmVarListReplaceStr(&Env->Vars, "ENV.QUERY_STRING", q->query_string);
+  }
+  else
+  {
+    udm_snprintf(q->query_string, sizeof(q->query_string), "q=");
+  }
+  return UDM_OK;
+}
+
+
+static int
+UdmDetectSelfTemplateQueryDaemon(UDM_ENV *Env,
+                                 UDM_SELF_TEMPLATE_QUERY *q,
+                                 FILE *stream)
+{
+  char str[1024];
+  while (fgets(str, sizeof(str), stream))
+  {
+    UdmRTrim(str, "\r\n");
+    if (!strncmp(str, "GET ", 4))
+    {
+      char *method, *uri, *lt;
+      if ((method= udm_strtok_r(str, " ", &lt)) &&
+          (uri= udm_strtok_r(NULL, " ", &lt)))
+      {
+        char *conf_dir, *query_string= strchr(uri, '?');
+        if (query_string)
+        {
+          *query_string= '\0';
+          query_string++;
+          udm_snprintf(q->query_string, sizeof(q->query_string), "%s", query_string);
+        }
+        conf_dir= UdmConfDir();
+        udm_snprintf(q->template_name, sizeof(q->template_name)-5, "%s%s%s", 
+                     conf_dir, UDMSLASHSTR, uri);
+        udm_snprintf(q->self, sizeof(q->self), "%s",  uri);
+        UDM_FREE(conf_dir);
+      }
+    }
+    if (!strcmp(str,""))
+      break;
+  }
+  return UDM_OK;
+}
+
+
 int main(int argc, char ** argv)
 {
   const char  *env, *bcharset, *lcharset;
-  char        template_name[1024]= "";
   char        search_time[100]= "";
-  char        self[1024]= "";
-  char        *query_string= NULL, *nav= NULL, *url= NULL;
+  char        *nav= NULL, *url= NULL;
   char        *searchwords= NULL, *storedstr= NULL;
-  int         page_size, page_number, res,httpd= 0;
+  int         page_size, page_number, res, ch, httpd= 0;
   size_t      i, swlen= 0, nav_len, storedlen, catcolumns= 0;
   ssize_t     page1,page2,npages,ppp=10;
   UDM_ENV     *Env;
@@ -343,14 +503,40 @@ int main(int argc, char ** argv)
   UDM_RESULT  *Res;
   UDM_VARLIST query_vars;
   UDM_VARLIST tmpl;
+  UDM_SELF_TEMPLATE_QUERY q;
+  
+  q.template_name[0]= 0;
+  q.self[0]= 0;
+  q.query_string[0]= 0;
+
+  while ((ch= getopt(argc, argv, "xh?")) != -1)
+  {
+    switch (ch)
+    {
+      case 'x':
+        httpd|= UDM_HTTPD_DAEMON;
+        break;
+      case '?':
+      case 'h':
+      default:
+        usage();
+        exit(0);
+    }
+  }
+  argc-= optind;
+  argv+= optind;
   
   UdmWSAStartup();
-  /* Output Content-type if under HTTPD   */
-  /* Some servers do not pass QUERY_STRING */
-  /* if the query was empty, so check   */
-  /* REQUEST_METHOD too     to be safe     */
   
-  httpd=(getenv("QUERY_STRING")||getenv("REQUEST_METHOD"));
+  /*
+    Output Content-type if under HTTPD
+    Some servers do not pass QUERY_STRING
+    if the query was empty, so check
+    REQUEST_METHOD too to be safe
+  */
+  
+  httpd|= getenv("QUERY_STRING") || getenv("REQUEST_METHOD") ?
+          UDM_HTTPD_CGI : 0;
 
 #if ( (defined WIN32) && (defined _DEBUG) )
   if(httpd)
@@ -374,113 +560,27 @@ int main(int argc, char ** argv)
   UdmVarListAddEnviron(&Env->Vars,"ENV");
   UdmVarListReplaceStr(&Env->Vars, "version", VERSION);
   
-  /* Detect self */
-  if((env= getenv("UDMSEARCH_SELF")) || (env= getenv("REQUEST_URI")))
-  {
-    /* Use Apache's REQUEST_URI, which is not in CGI standard */
-    char *end;
-    strncpy(self,env,sizeof(self)-1);
-    if ((end= strchr(self,'?')))
-      end[0]='\0';
-  }
-  else if ((env= getenv("SCRIPT_NAME")))
-  {
-    /* Use standard CGI variables if not under Apache */
-    const char *path_info= getenv("PATH_INFO");
-    udm_snprintf(self,sizeof(self),"%s%s",env,path_info?path_info:"");
-  }
+  if (httpd & UDM_HTTPD_DAEMON)
+    UdmDetectSelfTemplateQueryDaemon(Env, &q, stdin);
   else
-  {
-    strncpy(self,"search.cgi",sizeof(self)-1);
-  }
+    UdmDetectSelfTemplateQueryCGI(Env, &q, argc, argv);
   
   
-  /* Detect template name */
-  if((env= getenv("UDMSEARCH_TEMPLATE")))
-  {
-    char ename[1024];
-    strncpy(ename,env,sizeof(ename)-1);
-    ename[sizeof(ename)]='\0';
-    UdmUnescapeCGIQuery(template_name,ename);
-  }
-  else if(getenv("REDIRECT_STATUS") && (env= getenv("PATH_TRANSLATED")))
-  {
-    /* Check Apache internal redirect  */
-    /* via   "AddHandler" and "Action" */
-    strncpy(template_name,env,sizeof(template_name)-1);
-  }
-  else if ((env= getenv("PATH_INFO")) && env[0])
-  {
-    /* http://localhost/cgi-bin/search.cgi/path/to/search.htm */
-    char name[128], *conf_dir;
-    
-    strncpy(name,env,sizeof(name)-1);
-    name[sizeof(name)-1]='\0';
-    if (UDMSLASH !='/')
-    {
-      char *s;
-      for (s=name; s[0]; s++)
-        if (s[0]=='\\')s[0]='/';
-    }
-    
-    conf_dir= ((env= getenv("UDM_CONF_DIR")) || (env= getenv("UDM_ETC_DIR"))) ?
-               (char *)UdmStrdup(env) : UdmConfDir();
-    /* Take from the config directory */
-    udm_snprintf(template_name, sizeof(template_name)-5, "%s%s%s", 
-                 conf_dir,UDMSLASHSTR,name);
-    UDM_FREE(conf_dir);
-  }
-  else
-  {
-    /* CGI executed without Apache internal redirect */
-    /* or started from command line                  */
-    char *end, *conf_dir;
-    size_t length;
-    
-    conf_dir= ((env= getenv("UDM_CONF_DIR")) || (env= getenv("UDM_ETC_DIR"))) ?
-               (char *)UdmStrdup(env) : UdmConfDir();
-    
-    /* Take from the config directory */
-    length= udm_snprintf(template_name, sizeof(template_name)-5, "%s%s%s", 
-                         conf_dir,UDMSLASHSTR,
-                         (end= strrchr(self,'/')) ? (end+1) : (self));
-    end= template_name + length;
-    if ( length > 3 && (!strcmp(end-4,".exe") || !strcmp(end-4,".cgi")))
-      end-=4;
-    strcpy(end,".htm");
-    UDM_FREE(conf_dir);
-  }
-  
-  if((env= getenv("QUERY_STRING")))
-  {
-    query_string = (char*)UdmRealloc(query_string, strlen(env) + 2);
-    strncpy(query_string, env, strlen(env) + 1);
-  }
-  else if(argv[1])
-  {
-    query_string = (char*)UdmRealloc(query_string, strlen(argv[1]) + 10);
-    sprintf(query_string, "q=%s", argv[1]);
-    UdmVarListReplaceStr(&Env->Vars, "ENV.QUERY_STRING", query_string);
-  }
-  else
-  {
-    query_string = (char*)UdmRealloc(query_string, 1024);
-    sprintf(query_string, "q=");
-  }
-  
-  /* Hack for Russian Apache from apache.lexa.ru  */
-  /* QUERY_STRING is already converted to server  */
-  /* character set. We must print original query  */
-  /* string instead however. Under usual apache   */ 
-  /* we'll use QUERY_STRING. Note that query_vars */
-  /* list will contain not unescaped values, so   */
-  /* we don't have to escape them when displaying */
+  /*
+    Hack for Russian Apache from apache.lexa.ru
+    QUERY_STRING is already converted to server
+    character set. We must print original query
+    string instead however. Under usual apache
+    we'll use QUERY_STRING. Note that query_vars
+    list will contain not unescaped values, so
+    we don't have to escape them when displaying
+  */
   env= getenv("CHARSET_SAVED_QUERY_STRING");
-  ParseQStringUnescaped(&query_vars,env?env:query_string);
+  ParseQStringUnescaped(&query_vars, env ? env : q.query_string);
   
   /* Unescape and save variables from QUERY_STRING */
   /* Env->Vars will have unescaped values however  */
-  UdmParseQueryString(Agent,&Env->Vars,query_string);
+  UdmParseQueryString(Agent, &Env->Vars, q.query_string);
 
 #ifndef WIN32
   env= getenv("REMOTE_USER");
@@ -491,14 +591,13 @@ int main(int argc, char ** argv)
   }
 #endif
   
-  if((res=UdmTemplateLoad(Agent,template_name,&tmpl)))
+  if((res=UdmTemplateLoad(Agent, q.template_name, &tmpl)))
   {
     UdmHTTPHeadersSend(httpd, "text/plain", NULL);
     printf("%s\n",Env->errstr);
     UdmVarListFree(&tmpl);
     UdmVarListFree(&query_vars);
     UdmEnvFree(Env);
-    UDM_FREE(query_string);
     UdmAgentFree(Agent);
     return(0);
   }
@@ -525,16 +624,14 @@ int main(int argc, char ** argv)
   
   UdmTemplatePrint(Agent, NULL, NULL, 0, &Env->Vars, &tmpl, "variables");
 
-  /* Call again to load search Limits if need */
-/*  UdmParseQueryString(Agent, &Env->Vars, query_string); */
   UdmSetLogLevel(NULL, UdmVarListFindInt(&Env->Vars, "LogLevel", 0));
   UdmOpenLog("search.cgi", Env, !strcasecmp(UdmVarListFindStr(&Env->Vars, "Log2stderr", (!httpd) ? "yes" : "no"), "yes"));
-  UdmLog(Agent,UDM_LOG_ERROR,"search.cgi started with '%s'",template_name);
+  UdmLog(Agent,UDM_LOG_ERROR, "search.cgi started with '%s'", q.template_name);
   
   /* This is for query tracking */
-  UdmVarListReplaceStr(&Env->Vars, "tmplt", template_name);
-  UdmVarListAddStr(&Env->Vars,"QUERY_STRING",query_string);
-  UdmVarListAddStr(&Env->Vars,"self",self);
+  UdmVarListReplaceStr(&Env->Vars, "tmplt", q.template_name);
+  UdmVarListAddStr(&Env->Vars,"QUERY_STRING", q.query_string);
+  UdmVarListAddStr(&Env->Vars,"self", q.self);
   
   bcharset= UdmVarListFindStr(&Env->Vars,"BrowserCharset","iso-8859-1");
   Env->bcs= UdmGetCharSet(bcharset);
@@ -545,22 +642,23 @@ int main(int argc, char ** argv)
   if (! Env->bcs)
   {
     UdmHTTPHeadersSend(httpd, "text/plain", NULL);
-    printf("Unknown BrowserCharset '%s' in template '%s'\n",bcharset,template_name);
+    printf("Unknown BrowserCharset '%s' in template '%s'\n",
+           bcharset, q.template_name);
     exit(0);
   }
   
   if(! Env->lcs)
   {
     UdmHTTPHeadersSend(httpd, "text/plain", NULL);
-    printf("Unknown LocalCharset '%s' in template '%s'\n",lcharset,template_name);
+    printf("Unknown LocalCharset '%s' in template '%s'\n",
+           lcharset, q.template_name);
     exit(0);
   }
   
   if (UdmVarListFindInt(&Env->Vars, "cc", 0))
   {
     UdmVarListFree(&query_vars);
-    UDM_FREE(query_string);
-    return(StoreDocCGI(Agent, Env, &tmpl));
+    return StoreDocCGI(Agent, Env, &tmpl, httpd);
   }
   
   UdmHTTPHeadersSend(httpd,
@@ -572,89 +670,6 @@ int main(int argc, char ** argv)
   res= UdmVarListFindInt(&Env->Vars,"np",0)*UdmVarListFindInt(&Env->Vars,"ps",10);
   UdmVarListAddInt(&Env->Vars,"pn",res);
   
-  catcolumns= (size_t)atoi(UdmVarListFindStr(&Env->Vars,"CatColumns",""));
-  
-  if(catcolumns)
-  {
-    UDM_CATEGORY C;
-    
-    bzero((void*)&C, sizeof(C));
-    strcpy(C.addr,UdmVarListFindStr(&Env->Vars,"cat",""));
-    if(UDM_OK == UdmCatAction(Agent, &C, UDM_CAT_ACTION_LIST))
-    {
-      size_t n=1, l = 0;
-      char *catlist;
-      
-      for(i= 0; i < C.ncategories; i++)
-        l+= 128 + strlen(C.Category[i].path) + strlen(C.Category[i].name);
-      
-      if (l > 0  && (catlist= (char*)UdmMalloc(l)))
-      {
-        sprintf(catlist, "<table>\n");
-        for(i = 0; i < C.ncategories; i++)
-        {
-          sprintf(catlist+strlen(catlist),
-                  "%s<td><a href=\"?cat=%s\">%s</A></td><td width=60>&nbsp;</td>\n",
-                  n==1 ? "<tr>\n" : "",
-                  C.Category[i].path, C.Category[i].name);
-          
-          if(n == catcolumns)
-          {
-            sprintf(catlist+strlen(catlist),"</tr>\n");
-            n=1;
-          }
-          else
-          {
-            n++;
-          }
-        }
-        sprintf(catlist + strlen(catlist), "</table>\n");
-        UdmVarListAddStr(&Env->Vars, "CS", catlist);
-        UDM_FREE(catlist);
-      }
-    }
-    else
-    {
-      UdmVarListAddStr(&Env->Vars,"E",UdmEnvErrMsg(Agent->Conf));
-      UdmTemplatePrint(Agent, stdout, NULL, 0, &Env->Vars, &tmpl, "top");
-      UdmTemplatePrint(Agent, stdout, NULL, 0, &Env->Vars, &tmpl, "error");
-      goto end;
-    }
-    
-    UDM_FREE(C.Category);
-    bzero((void*)&C, sizeof(C));
-    strcpy(C.addr,UdmVarListFindStr(&Env->Vars,"cat",""));
-    if(UDM_OK == UdmCatAction(Agent, &C, UDM_CAT_ACTION_PATH))
-    {
-      char *catpath = NULL;
-      size_t l= 0;
-      
-      for(i= 0; i < C.ncategories; i++)
-        l+= 32 + strlen(C.Category[i].path) + strlen(C.Category[i].name);
-      
-      if (l > 0 && (catpath= (char*)UdmMalloc(l)))
-      {
-        catpath[0]= '\0';
-        for(i = 0; i < C.ncategories; i++)
-        {
-          sprintf(catpath+strlen(catpath),"/<a href=\"?cat=%s\">%s</A>",
-                  (C.Category[i].path) ? C.Category[i].path : "",
-                  (C.Category[i].name) ? C.Category[i].name : "");
-        }
-        UdmVarListAddStr(&Env->Vars, "CP", catpath);
-        UDM_FREE(catpath);
-      }
-    }
-    else
-    {
-      UdmVarListAddStr(&Env->Vars,"E",UdmEnvErrMsg(Agent->Conf));
-      UdmTemplatePrint(Agent, stdout, NULL, 0, &Env->Vars, &tmpl, "top");
-      UdmTemplatePrint(Agent, stdout, NULL, 0, &Env->Vars, &tmpl, "error");
-      goto end;
-    }
-    UDM_FREE(C.Category);
-  }
-  
   if(NULL==(Res=UdmFind(Agent)))
   {
     UdmVarListAddStr(&Env->Vars,"E",UdmEnvErrMsg(Agent->Conf));
@@ -872,7 +887,6 @@ end:
   UdmVarListFree(&query_vars);
   UdmEnvFree(Env);
   UdmAgentFree(Agent);
-  UDM_FREE(query_string);
   UDM_FREE(url);
   UDM_FREE(nav);
   if (httpd) 
