@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,12 +54,12 @@ static void cache_file_name(char *dst,size_t len,UDM_ENV *Conf,UDM_RESULT *Res){
 /*	UDM_DB *db = Conf->db;*/
 	
 	bytes = udm_snprintf(param, sizeof(param)-1,
-		"%s.%s.%s.%s.%s.%d.%s.%s.%s.%s.%s.%s.%s.%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s-%s",
+		"%s.%s.%s.%s.%s.%s.%s.%s.%s.%s.%s.%s.%s.%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s-%s",
 			     UdmVarListFindStr(&Conf->Vars,"SearchMode","all"),
 			     UdmVarListFindStr(&Conf->Vars, "orig_m", ""),
 			     UdmVarListFindStr(&Conf->Vars, "fl", ""),
 			     UdmVarListFindStr(&Conf->Vars, "wm", ""),
-			     UdmVarListFindInt(&Conf->Vars, "o", 0),
+			     UdmVarListFindStr(&Conf->Vars, "o", ""),
 			     UdmVarListFindStr(&Conf->Vars, "t", ""),
 			     UdmVarListFindStr(&Conf->Vars, "cat", ""),
 			     UdmVarListFindStr(&Conf->Vars, "ul", ""),
@@ -161,6 +161,8 @@ free:
 
 
 int __UDMCALL UdmSearchCacheStore(UDM_AGENT * query,UDM_RESULT *Res){
+	return 0;
+#if 0
 	int	fd;
 	char	fname[1024];
 	size_t	i;
@@ -203,9 +205,12 @@ int __UDMCALL UdmSearchCacheStore(UDM_AGENT * query,UDM_RESULT *Res){
 		}
 	}
 	return(0);
+#endif
 }
 
 int __UDMCALL UdmSearchCacheFind(UDM_AGENT * Agent,UDM_RESULT *Res){
+	return -1;
+#if 0
 	char fname[1024];
 	int fd;
 	UDM_URL_CRD *wrd = NULL;
@@ -305,4 +310,5 @@ int __UDMCALL UdmSearchCacheFind(UDM_AGENT * Agent,UDM_RESULT *Res){
 #endif
 
 	return(0);
+#endif
 }
