@@ -207,7 +207,7 @@ UdmStackItemListAddWord(UDM_AGENT *query, UDM_RESULT *Res,
     Substring searches should not exclude them.
   */
   if(final_word_match == UDM_MATCH_FULL &&
-     (UdmStopListFind(&query->Conf->StopWords, wrd, state->lang) ||
+     (UdmStopListListFind(&query->Conf->StopWord, wrd, state->lang) ||
       query->Conf->WordParam.min_word_len > length ||
       query->Conf->WordParam.max_word_len < length))
   {
@@ -505,7 +505,8 @@ void UdmDecodeHex8Str(const char *hex_str, uint4 *hi,
 
 
 int UdmParseQueryString(UDM_AGENT * Agent,
-                        UDM_VARLIST * vars,char * query_string)
+                        UDM_VARLIST * vars,
+                        const char * query_string)
 {
   char * tok, *lt;
   size_t len;
