@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -322,18 +322,52 @@ static const struct udm_sgml_chars
 {    "",       0    }};  
 
 
-int UdmSgmlToUni(const char *sgml){
-     int i;
-     int res;
-
-     for(i=0;SGMLChars[i].unicode;i++)
-          if(!strncmp(sgml,SGMLChars[i].sgml,strlen(SGMLChars[i].sgml))){
-               res=SGMLChars[i].unicode;
-               return(res);
-          }
-     return(0);
+int UdmSgmlToUni(const char *sgml)
+{
+  const struct udm_sgml_chars *p;
+  for(p= SGMLChars; p->unicode; p++)
+  {
+    const char *s, *t;
+    for (s= sgml, t= p->sgml; *s == *t ; s++, t++);
+    if (!*t)
+      return p->unicode;
+  }
+  return 0;
 }
 
+
+/*
+  Scan an entity. Return number of bytes scanned.
+*/
+int UdmSGMLScan(int *wc, const unsigned char *str, const unsigned char *end)
+{
+  const unsigned char *p, *end10= str + 10;
+  if (end10 > end)
+    end10= end;
+  for (p= str + 2; p < end10; p++)
+  {
+    if (*p == ';')
+    {
+      if (str[1] == '#')
+      {
+        if (str[2] == 'x' || str[2] == 'X')
+          *wc= strtol((const char*)str + 3, NULL, 16);
+        else
+          *wc= strtol((const char*)str + 2, NULL, 10);
+      }
+      else
+      {
+        *wc= UdmSgmlToUni((const char*) str + 1);
+      }
+      if (*wc)
+        return p - str + 1;
+    }
+  }
+  *wc= '&';
+  return 1;
+}
+
+
 /** This function replaces SGML entities
     With their character equivalents     
 */
