@@ -141,7 +141,7 @@ UdmSpellListLoad(UDM_SPELLLIST *L, char *err, size_t errlen)
 
   for (i=0 ; i < 256; i++)
     tolowermap[i]= i;
-  L->cs->lcase(L->cs, tolowermap, 256);
+  L->cs->lcase(udm_unidata_default, L->cs, tolowermap, 256);
 
   for (tok= L->fbody; *tok; )
   {
@@ -199,11 +199,12 @@ UdmSpellListLoad(UDM_SPELLLIST *L, char *err, size_t errlen)
     L->cs->lcase(L->cs, S->word, wordlen);
 #endif
 
-    L->nitems++;
+    if (S->word[0] != '#')
+      L->nitems++;
   }
 
   if (!strstr(filename, "sorted"))
-    qsort((void*)L->Item, L->nitems, sizeof(L->Item[0]), cmpspell_word_and_flag);
+    UdmSort((void*)L->Item, L->nitems, sizeof(L->Item[0]), cmpspell_word_and_flag);
 
 ex:
 
@@ -334,6 +335,11 @@ UdmSpellListListWriteHash(UDM_SPELLLISTLIST *L, char *err, size_t errlen)
 {
   int rc= UDM_OK;
   size_t i;
+  if (!L->nitems)
+  {
+    udm_snprintf(err, errlen, "No ispell word lists were loaded, nothing to hash");
+    return UDM_ERROR;
+  }
   for (i= 0; i < L->nitems; i++)
   {
     if (UDM_OK != (rc= UdmSpellListWriteHash(&L->Item[i], err, errlen)))
@@ -466,7 +472,7 @@ static void rmsp(char *s)
 static
 int UdmAffixListLoad(UDM_AFFIXLIST *L, int flags, char *err, size_t errlen)
 {
-  char str[128];
+  char strbuf[128];
   char flag= 0;
   char mask[1024]= "";
   char find[1024]= "";
@@ -495,9 +501,11 @@ int UdmAffixListLoad(UDM_AFFIXLIST *L, int flags, char *err, size_t errlen)
     return UDM_ERROR;
   }
 
-  while(fgets(str,sizeof(str),affix))
+  while(fgets(strbuf, sizeof(strbuf), affix))
   {
     UDM_AFFIX *A;
+    char *str= UdmTrim(strbuf, "\t ");
+
     if(!strncasecmp(str,"suffixes",8))
     {
       suffixes=1;
@@ -530,13 +538,13 @@ int UdmAffixListLoad(UDM_AFFIXLIST *L, int flags, char *err, size_t errlen)
     i= sscanf(str,"%[^>\n]>%[^,\n],%[^\n]",mask,find,repl);
 
     rmsp(repl);
-    UdmStrToLower(L->cs, repl, strlen(repl));
+    UdmStrToLower(udm_unidata_default, L->cs, repl, strlen(repl));
     
     rmsp(find);
-    UdmStrToLower(L->cs, find, strlen(find));
+    UdmStrToLower(udm_unidata_default, L->cs, find, strlen(find));
     
     rmsp(mask);
-    UdmStrToLower(L->cs, mask, strlen(mask));
+    UdmStrToLower(udm_unidata_default, L->cs, mask, strlen(mask));
 
     switch(i)
     {
@@ -695,9 +703,9 @@ typedef struct udm_spell_search_info_st
 static UDM_SPELL *
 UdmSpellListFindFirst(UDM_SPELLLIST *Sl, UDM_SPELL *Find, UDM_SPSEARCH_INFO *Si)
 {
-  UDM_SPELL *Beg= (UDM_SPELL*) bsearch((const void *) Find,
-                                       (const void *) Sl->Item, Sl->nitems,
-                                       sizeof(Sl->Item[0]), cmpspell);
+  UDM_SPELL *Beg= (UDM_SPELL*) UdmBSearch((const void *) Find,
+                                          (const void *) Sl->Item, Sl->nitems,
+                                           sizeof(Sl->Item[0]), cmpspell);
   for ( ; Beg && Beg > Sl->Item && !strcmp(Find->word, Beg[-1].word); Beg--);
   return Beg;
 }
@@ -706,9 +714,9 @@ UdmSpellListFindFirst(UDM_SPELLLIST *Sl, UDM_SPELL *Find, UDM_SPSEARCH_INFO *Si)
 static UDM_SPELL *
 UdmSpellListFindNext(UDM_SPELLLIST *Sl, UDM_SPELL *Prev, UDM_SPSEARCH_INFO *Si)
 {
-  UDM_SPELL *Next= Prev + 1;
-  if (Next < &Sl->Item[Sl->nitems] && !strcmp(Prev->word, Next->word))
-    return Next;
+  Si->pos++;
+  if (Si->pos < Sl->nitems && !strcmp(Prev->word, Sl->Item[Si->pos].word))
+    return &Sl->Item[Si->pos];
   return NULL;
 }
 
@@ -1011,7 +1019,8 @@ static UDM_ENV Env;
                    can't be opened, or out of memory
 */
 
-static int udm_parser_init(void)
+static int
+udm_parser_init(void *param __attribute__((unused)))
 {
   const char *conf= "stemming.conf";
   char err[128]= "";
@@ -1064,7 +1073,8 @@ ex:
     0                    never fails
 */
 
-static int udm_parser_deinit(void)
+static int
+udm_parser_deinit(void *param __attribute__((unused)))
 {
   UdmEnvFree(&Env);
   return 0;
@@ -1101,7 +1111,7 @@ add_base_form(MYSQL_FTPARSER_PARAM *param, char *word, size_t length,
   if (length >= sizeof(tmpword))
   {
     /* Too long word, don't try to normalize */
-    param->mysql_add_word(param->mysql_ftparam, word, length, boolean_info);
+    param->mysql_add_word(param, word, length, boolean_info);
     return;
   }
   
@@ -1117,27 +1127,28 @@ add_base_form(MYSQL_FTPARSER_PARAM *param, char *word, size_t length,
     if (param->mode == MYSQL_FTPARSER_FULL_BOOLEAN_INFO)
     {
       boolean_info->type= FT_TOKEN_LEFT_PAREN;
-      param->mysql_add_word(param->mysql_ftparam, 0, 0, boolean_info);
+      param->mysql_add_word(param, 0, 0, boolean_info);
       boolean_info->type= FT_TOKEN_WORD;
+      boolean_info->yesno= 0;
     }
     for (N= Norm ; N < Norm + nforms; N++)
     {
       size_t nlength= strlen(N->word);
       if (nlength >= Env.WordParam.min_word_len)
-        param->mysql_add_word(param->mysql_ftparam, Norm->word, nlength,
-                              boolean_info);
+        param->mysql_add_word(param, N->word, nlength, boolean_info);
     }
     if (param->mode == MYSQL_FTPARSER_FULL_BOOLEAN_INFO)
     {
       boolean_info->type= FT_TOKEN_RIGHT_PAREN;
-      param->mysql_add_word(param->mysql_ftparam, 0, 0, boolean_info);
+      param->mysql_add_word(param, 0, 0, boolean_info);
       boolean_info->type= FT_TOKEN_WORD;
+      boolean_info->yesno= 1;
     }
   }
   else
   {
     /* No forms found, add the word itself */
-    param->mysql_add_word(param->mysql_ftparam, word, length, boolean_info);
+    param->mysql_add_word(param, word, length, boolean_info);
   }
 
 #if 0
@@ -1241,21 +1252,27 @@ udm_parser_parse(MYSQL_FTPARSER_PARAM *param)
 
 static struct st_mysql_ftparser udm_parser_declaration=
 {
-  MYSQL_FTPARSER_INTERFACE_VERSION, udm_parser_parse, 0, 0
+  MYSQL_FTPARSER_INTERFACE_VERSION, /* interface version      */
+  udm_parser_parse,                 /* parsing function       */
+  0,                                /* parser init function   */
+  0                                 /* parser deinit function */
 };
 
 
-mysql_declare_plugin
+mysql_declare_plugin(stemming)
 {
-  MYSQL_FTPARSER_PLUGIN,
-  &udm_parser_declaration,
-  "stemming",
-  "mnoGoSearch developers team",
-  "Fulltext pre-parser with ispell-based stemming",
-  udm_parser_init,
-  udm_parser_deinit,
-  0, /* version */
-  0 /* status vars */
+  MYSQL_FTPARSER_PLUGIN,      /* type                            */
+  &udm_parser_declaration,    /* descriptor                      */
+  "stemming",                 /* name                            */
+  "mnoGoSearch team",         /* author                          */
+  "ispell-based stemming",    /* description                     */
+  PLUGIN_LICENSE_GPL,
+  udm_parser_init,            /* init function (when loaded)     */
+  udm_parser_deinit,          /* deinit function (when unloaded) */
+  0x0001,                     /* version                         */
+  NULL,                       /* status variables                */
+  NULL,                       /* system variables                */
+  NULL                        /* config options                  */
 }
 mysql_declare_plugin_end;
 
