@@ -271,14 +271,13 @@ UdmIBExecInternal(UDM_DB *db, UDM_SQLRES *res)
     
     for (i = 0; i < osqlda->sqld; i++)
     {
-      XSQLVAR *var=&osqlda->sqlvar[i];
-      int coltype = osqlda->sqlvar[i].sqltype & ~1;
-      
-      osqlda->sqlvar[i].sqlind=&sqlind_array[i];
-      osqlda->sqlvar[i].sqlind[0]=0;
-      res->Fields[i].sqlname = (char*)UdmStrdup(osqlda->sqlvar[i].sqlname);
-      res->Fields[i].sqllen  = osqlda->sqlvar[i].sqllen;
+      XSQLVAR *var= &osqlda->sqlvar[i];
+      int coltype= osqlda->sqlvar[i].sqltype & ~1;
       
+      var->sqlind= &sqlind_array[i];
+      var->sqlind[0]= 0;
+      res->Fields[i].sqlname= (char*)UdmStrdup(var->aliasname[0] ? var->aliasname : var->sqlname);
+      res->Fields[i].sqllen= var->sqllen;
       
       switch(coltype)
       {
