@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 #define CR_SERVER_LOST          2013
 #define CR_SERVER_GONE_ERROR    2006
 #define ER_SERVER_SHUTDOWN      1053
-
+#define ER_DUP_ENTRY_WITH_KEY_NAME 1582
 
 /*
   Return MySQL version as number: 4.1.24 -> 40124
@@ -66,12 +66,14 @@ static int udm_mysql_version(MYSQL *mysql)
 static int UdmMySQLInit(UDM_DB *db)
 {
   const char* DBLog=  UdmVarListFindStr(&db->Vars,"sqllog",NULL);
+  const char* DBLogBin= UdmVarListFindStr(&db->Vars,"sqllogbin",NULL);
   const char* DBSock= UdmVarListFindStr(&db->Vars,"socket",NULL);
   const char* DBUser= UdmVarListFindStr(&db->Vars,"DBUser",NULL);
   const char* DBPass= UdmVarListFindStr(&db->Vars,"DBPass",NULL);
   const char* DBHost= UdmVarListFindStr(&db->Vars, "DBHost", "localhost");
   const char* mycnfgroup= UdmVarListFindStr(&db->Vars, "mycnfgroup", "client");
   int DBPort= UdmVarListFindInt(&db->Vars, "DBPort", 0);
+  const char* setnames=  UdmVarListFindStr(&db->Vars,"setnames", "");
   MYSQL *mysql;
   
   db->specific = mysql = mysql_init(NULL);
@@ -97,12 +99,44 @@ static int UdmMySQLInit(UDM_DB *db)
   db->connected=1;
   db->version= udm_mysql_version(mysql);
   if (db->version >= 40100)
+  {
     db->flags|= UDM_SQL_HAVE_SUBSELECT;
+    if (setnames[0])
+    {
+      char qbuf[64];
+      udm_snprintf(qbuf, sizeof(qbuf), "SET NAMES %s", setnames);
+      if (mysql_query(mysql, qbuf))
+      {
+        db->errcode=1;
+        sprintf(db->errstr,"MySQL driver: in SET NAMES: #%d: %s",
+                mysql_errno(mysql),mysql_error(mysql));
+        return UDM_ERROR;
+      }
+    }
+  }
   if (DBLog)
   {
     char qbuf[64];
     sprintf(qbuf, "SET SQL_LOG_OFF=%d", atoi(DBLog) ? 0 : 1);
-    mysql_query(mysql, qbuf);
+    if (mysql_query(mysql, qbuf))
+    {
+      db->errcode=1;
+      sprintf(db->errstr,"MySQL driver: in SET SQL_LOG_OFF: #%d: %s",
+              mysql_errno(mysql),mysql_error(mysql));
+      return UDM_ERROR;
+    }
+  }
+  if (DBLogBin)
+  {
+    char qbuf[64];
+    sprintf(qbuf, "SET SQL_LOG_BIN=%d", atoi(DBLogBin) ? 1 : 0);
+    if (mysql_query(mysql, qbuf))
+    {
+      db->errcode=1;
+      sprintf(db->errstr,"MySQL driver: in SET SQL_LOG_BIN: #%d: %s",
+              mysql_errno(mysql),mysql_error(mysql));
+      return UDM_ERROR;
+    }
   }
   return UDM_OK;
 }
@@ -199,7 +233,8 @@ static int UdmMySQLExecDirect(UDM_DB *db,UDM_SQLRES *R,const char *query)
         sprintf(db->errstr,"MySQL driver: #%d: %s",
                 mysql_errno(mysql),mysql_error(mysql));
         if((mysql_errno(mysql)!=ER_DUP_ENTRY) &&
-           (mysql_errno(mysql)!=ER_DUP_KEY))
+           (mysql_errno(mysql)!=ER_DUP_KEY) &&
+           (mysql_errno(mysql)!=ER_DUP_ENTRY_WITH_KEY_NAME))
         {
           db->errcode=1;
           return UDM_ERROR;
