@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,7 +67,8 @@ static int udm_oracle_types[] =
   SQLT_LNG
 };
 
-static int UdmOracle8DisplayError(UDM_DB *db)
+static int
+UdmOracle8DisplayError(UDM_DB *db, const char *fname)
 {
   sb4  errcode=0;
   char  errbuf[512];
@@ -100,7 +101,8 @@ static int UdmOracle8DisplayError(UDM_DB *db)
           *ptr='!';
         ++ptr;
       }
-      sprintf(db->errstr,"Oracle - %.*s", 512, errbuf);
+      udm_snprintf(db->errstr, sizeof(db->errstr) - 1,
+                   "Oracle: %s: %.*s", fname, 512, errbuf);
       break;
     case OCI_INVALID_HANDLE:
       sprintf(db->errstr,"Oracle - OCI_INVALID_HANDLE");
@@ -508,23 +510,41 @@ static int UdmOracle8Query(UDM_DB *db, UDM_SQLRES *res, const char *qbuf)
 
   if(!db->connected)
   {
-    UdmOracle8InitDB(db);
-    if(db->errcode)
+    size_t i;
+    for (i= 0; i < 3; i++)
     {
-      UdmOracle8DisplayError(db);
-      return(UDM_ERROR);
-    }
-    else
-    {
-      db->connected=1;
+      UdmOracle8InitDB(db);
+      if(db->errcode)
+      {
+        UdmOracle8DisplayError(db, "InitDB");
+        if (strstr(db->errstr, "ORA-12520"))
+        {
+          /* No listener available, try again */
+          db->errcode= 0;
+          db->errstr[0]= '\0';
+          UDMSLEEP(1);
+          continue;
+        }
+        return(UDM_ERROR);
+      }
+      else
+      {
+        db->connected=1;
+        goto query;
+      }
     }
+    if (!db->connected)
+      goto error;
   }
-  
-    
-  rc = sql_oracle_query(db, res, qbuf);
+
+
+query:
+  rc= sql_oracle_query(db, res, qbuf);
+
+error:
   if(db->errcode)
   {
-    UdmOracle8DisplayError(db);
+    UdmOracle8DisplayError(db, "ExecDirect");
     return(UDM_ERROR);
   }
   return(rc);
@@ -541,7 +561,7 @@ static int UdmOracle8Prepare (UDM_DB *db, const char *query)
   if (! SQL_OK(_))
   {
     db->errcode = _;
-    UdmOracle8DisplayError(db);
+    UdmOracle8DisplayError(db, "Prepare");
     return(UDM_ERROR);
   }
 
@@ -550,7 +570,7 @@ static int UdmOracle8Prepare (UDM_DB *db, const char *query)
   if (! SQL_OK(_))
   {
     db->errcode = _;
-    UdmOracle8DisplayError(db);
+    UdmOracle8DisplayError(db, "Prepare");
     return(UDM_ERROR);
   }
   return(UDM_OK);
@@ -576,7 +596,7 @@ static int UdmOracle8Bind (UDM_DB *db, int position,
   if (! SQL_OK(_))
   {
     db->errcode = _;
-    UdmOracle8DisplayError(db);
+    UdmOracle8DisplayError(db, "Bind");
     return(UDM_ERROR);
   }
   return(UDM_OK);
@@ -594,7 +614,7 @@ static int UdmOracle8Exec (UDM_DB *db)
   if (! SQL_OK(_))
   {
     db->errcode = _;
-    UdmOracle8DisplayError(db);
+    UdmOracle8DisplayError(db, "Exec");
     return(UDM_ERROR);
   }
   
@@ -602,7 +622,7 @@ static int UdmOracle8Exec (UDM_DB *db)
   if (! SQL_OK(_))
   {
     db->errcode = _;
-    UdmOracle8DisplayError(db);
+    UdmOracle8DisplayError(db, "Exec");
     return(UDM_ERROR);
   }
   return(UDM_OK);
