@@ -38,14 +38,6 @@ void UdmSynonymListInit(UDM_SYNONYMLIST * List)
 }
 
 
-static int
-IntSet(int *dst, int src)
-{
-  *dst= src;
-  return 1;
-}
-
-
 static void
 AddSynonym(UDM_SYNONYMLIST *Syn, char *first, char *second)
 {
@@ -55,6 +47,12 @@ AddSynonym(UDM_SYNONYMLIST *Syn, char *first, char *second)
   Syn->nsynonyms++;
 }
 
+
+#define UDM_SYN_MODE_ONEWAY    0
+#define UDM_SYN_MODE_ROUNDTRIP 1
+#define UDM_SYN_MODE_RETURN    2
+
+
 __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
 {
   FILE         *f;
@@ -62,8 +60,11 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
   char         lang[64]="";
   UDM_CHARSET  *cs=NULL;
   UDM_CONV     file_lcs;
-  int          reverse= 1;
+  int          mode= UDM_SYN_MODE_ROUNDTRIP;
   UDM_UNIDATA  *unidata= Env->unidata;
+  UDM_SYNONYMLIST Synonyms;
+  
+  UdmSynonymListInit(&Synonyms);
   
   if(!(f=fopen(filename,"r")))
   {
@@ -101,15 +102,28 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
     else if (!strncasecmp(str, "Mode:", 5))
     {
       char *lasttok, *l;
-      if(!(l = udm_strtok_r(str + 5, " \t\n\r", &lasttok)) ||
-          ((IntSet(&reverse, 0) && strcasecmp(l, "oneway")) &&
-          (IntSet(&reverse, 1) && strcasecmp(l, "reverse"))))
+      if ((l= udm_strtok_r(str + 5, " \t\n\r", &lasttok)))
       {
-        udm_snprintf(Env->errstr, sizeof(Env->errstr),
-                     "Bad Mode command in synonym file '%s'", filename);
-        fclose(f);
-        return UDM_ERROR;
+        if (!strcasecmp(l, "oneway"))
+        {
+          mode= UDM_SYN_MODE_ONEWAY;
+          continue;
+        }
+        else if (!strcasecmp(l, "reverse") || !strcasecmp(l, "roundtrip"))
+        {
+          mode= UDM_SYN_MODE_ROUNDTRIP;
+          continue;
+        }
+        else if (!strcasecmp(l, "return"))
+        {
+          mode= UDM_SYN_MODE_RETURN;
+          continue;
+        }
       }
+      udm_snprintf(Env->errstr, sizeof(Env->errstr),
+                   "Bad Mode command in synonym file '%s'", filename);
+      fclose(f);
+      return UDM_ERROR;
     }
     else
     {
@@ -138,28 +152,35 @@ __C_LINK int __UDMCALL UdmSynonymListLoad(UDM_ENV * Env,const char * filename)
       if ((ac= UdmGetArgs(tmp, av, 255)) < 2)
         continue;
 
-      for (i = 0; i < (reverse ? (ac - 1) : 1) ; i++)
+      for (i = 0; i < (mode == UDM_SYN_MODE_RETURN ? ac - 1 : 1) ; i++)
       {
         for (j = i + 1; j < ac; j++)
         {
-          if((Env->Synonyms.nsynonyms + 1) >= Env->Synonyms.msynonyms)
+          if((Synonyms.nsynonyms + 1) >= Synonyms.msynonyms)
           {
-            Env->Synonyms.msynonyms+= 64;
-            Env->Synonyms.Synonym= (UDM_SYNONYM*)UdmRealloc(Env->Synonyms.Synonym, 
-                                    sizeof(UDM_SYNONYM)*Env->Synonyms.msynonyms);
+            Synonyms.msynonyms+= 64;
+            {
+              size_t nbytes= sizeof(UDM_SYNONYM)*Synonyms.msynonyms;
+              Synonyms.Synonym= (UDM_SYNONYM*)UdmRealloc(Synonyms.Synonym, nbytes);
+            }
           }
 
           /* Add direct order */
-          AddSynonym(&Env->Synonyms, av[i], av[j]);
+          if (mode == UDM_SYN_MODE_ONEWAY || mode == UDM_SYN_MODE_ROUNDTRIP)
+            AddSynonym(&Synonyms, av[i], av[j]);
           
           /* Add reverse order */
-          if (reverse)
-            AddSynonym(&Env->Synonyms, av[j], av[i]);
+          if (mode == UDM_SYN_MODE_RETURN || mode == UDM_SYN_MODE_ROUNDTRIP)
+            AddSynonym(&Synonyms, av[j], av[i]);
         }
       }
     }
   }
   fclose(f);
+  udm_snprintf(Synonyms.fname, sizeof(Synonyms.fname), "%s", filename);
+  udm_snprintf(Synonyms.cset, sizeof(Synonyms.cset), "%s", cs->name);
+  udm_snprintf(Synonyms.lang, sizeof(Synonyms.lang), "%s", lang);
+  UdmSynonymListListAdd(&Env->Synonym, &Synonyms);
   return UDM_OK;
 }
 
@@ -189,11 +210,11 @@ __C_LINK void __UDMCALL UdmSynonymListSort(UDM_SYNONYMLIST * List)
 }
 
 
-UDM_WIDEWORDLIST *
-UdmSynonymListFind(const UDM_SYNONYMLIST * List, UDM_WIDEWORD * wword)
+static UDM_WIDEWORDLIST *
+UdmSynonymListFind(UDM_WIDEWORDLIST *Res,
+                   const UDM_SYNONYMLIST *List, UDM_WIDEWORD *wword)
 {
   UDM_SYNONYM syn, *res, *first, *last;
-  UDM_WIDEWORDLIST *Res= NULL;
   
   if(!List->nsynonyms)
     return NULL;
@@ -206,9 +227,6 @@ UdmSynonymListFind(const UDM_SYNONYMLIST * List, UDM_WIDEWORD * wword)
     UDM_WIDEWORD ww= wword[0];
     ww.origin= UDM_WORD_ORIGIN_SYNONYM;
     
-    Res=(UDM_WIDEWORDLIST *)UdmMalloc(sizeof(*Res));
-    UdmWideWordListInit(Res);
-
     /* Find first and last synonym */
     for(first= res;
         first >= List->Synonym && !strcmp(wword->word, first->p);
@@ -252,3 +270,59 @@ UdmSynonymListFind(const UDM_SYNONYMLIST * List, UDM_WIDEWORD * wword)
   }
   return Res;
 }
+
+
+void
+UdmSynonymListListInit(UDM_SYNONYMLISTLIST *Lst)
+{
+  bzero((void*)Lst, sizeof(Lst[0]));
+}
+
+
+void
+UdmSynonymListListFree(UDM_SYNONYMLISTLIST *Lst)
+{
+  size_t i;
+  for (i= 0; i < Lst->nitems; i++)
+    UdmSynonymListFree(&Lst->Item[i]);
+  UDM_FREE(Lst->Item);
+}
+
+
+int
+UdmSynonymListListAdd(UDM_SYNONYMLISTLIST *Lst, UDM_SYNONYMLIST *Item)
+{
+  size_t nbytes= (Lst->nitems + 1) * sizeof(UDM_SYNONYMLIST);
+  if (!(Lst->Item= (UDM_SYNONYMLIST*) UdmRealloc(Lst->Item, nbytes)))
+    return UDM_ERROR;
+  Lst->Item[Lst->nitems++]= Item[0];
+  return UDM_OK;
+}
+
+
+__C_LINK void __UDMCALL UdmSynonymListListSortItems(UDM_SYNONYMLISTLIST *List)
+{
+  size_t i;
+  for (i= 0; i < List->nitems; i++)
+    UdmSynonymListSort(&List->Item[i]);
+}
+
+
+UDM_WIDEWORDLIST*
+UdmSynonymListListFind(const UDM_SYNONYMLISTLIST *SLL, UDM_WIDEWORD *word)
+{
+  size_t i;
+  UDM_WIDEWORDLIST *Res= (UDM_WIDEWORDLIST *) UdmMalloc(sizeof(*Res));
+  UdmWideWordListInit(Res);
+  
+  for (i= 0; i < SLL->nitems; i++)
+    UdmSynonymListFind(Res, &SLL->Item[i], word);
+  
+  if (!Res->nwords)
+  {
+    UdmWideWordListFree(Res);
+    UdmFree(Res);
+    return NULL;
+  }
+  return Res;
+}
