@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,36 +18,44 @@
 #include "udm_config.h"
 #include <stdio.h>
 #include "udm_uniconv.h"
-
+#include <assert.h>
 
 void __UDMCALL UdmConvInit(UDM_CONV *cnv,UDM_CHARSET *from,UDM_CHARSET *to, int fl)
 {
   cnv->from= from;
   cnv->to= to;
   cnv->flags= fl;
-  cnv->ibytes= 0;
-  cnv->obytes= 0;
-  cnv->icodes= cnv->ocodes = 1; /* default for old functions */
-  cnv->istate= cnv->ostate = 0; /* default state */
 }
 
+
 int __UDMCALL UdmConv(UDM_CONV *c, char *d, size_t dlen,const char *s, size_t slen)
 {
-  size_t     i, codes;
   int        res, wc[16]; /* is 16 is enough? */
-  int        zero= 0;
   char	     *d_o= d;
   const char *s_e= s+slen;
   char	     *d_e= d+dlen;
-  const char *s_o= s;
+  int        (*mb_wc)(struct udm_conv_st *conv_arg,
+                      struct udm_cset_st *cs_arg, int *wc_arg,
+                      const unsigned char *s_arg,
+                      const unsigned char *e_arg);
+  int        (*wc_mb)(struct udm_conv_st *conv_arg,
+                      struct udm_cset_st *cs, int *wc_arg,
+                      unsigned char *s_arg,
+                      unsigned char *e_arg);
+/*
+if (slen && s[0] != 0)
+  fprintf(stderr, "> %s %s %d '%.*s'\n", c->from->name, c->to->name, slen, slen, s);
+*/
   
-  c->istate = 0; /* set default state */
-  c->ostate = 0; /* set default state */
+  c->istate= 0; /* set default state */
+  c->ostate= 0; /* set default state */
+  mb_wc= c->from->mb_wc;
+  wc_mb= c->to->wc_mb;
   
   while(s <s_e && d <d_e)
   {
-    res= c->from->mb_wc(c, c->from, wc, (const unsigned char*)s,
-                                        (const unsigned char*)s_e);
+    res= mb_wc(c, c->from, wc, (const unsigned char*)s,
+                               (const unsigned char*)s_e);
     if (res > 0)
     {
       s+=res;
@@ -59,60 +67,54 @@ int __UDMCALL UdmConv(UDM_CONV *c, char *d, size_t dlen,const char *s, size_t sl
              (res == UDM_CHARSET_ILSEQ5) ||
              (res == UDM_CHARSET_ILSEQ6))
     {
-      switch (res)
-      {
-        case UDM_CHARSET_ILSEQ6: s++;
-        case UDM_CHARSET_ILSEQ5: s++;
-        case UDM_CHARSET_ILSEQ4: s++;
-        case UDM_CHARSET_ILSEQ3: s++;
-        case UDM_CHARSET_ILSEQ2: s++;
-        case UDM_CHARSET_ILSEQ:
-        default: s++; break;
-      }
-      wc[0] = '?';
-    }
-    else
-      break;
-    
-    codes= c->ocodes;
-    for(i= 0; i < codes; i+= c->icodes)
-    {
-outp:
-      if (wc[i] == 0)
+      /* Move at least one character */
+      s+= ((res == UDM_CHARSET_ILSEQ) ? 1 : - res);
+      wc[0]= '?';
+      res= wc_mb(c, c->to, &wc[0], (unsigned char*) d, (unsigned char*) d_e);
+      if (res <= 0)
         goto outaway;
-      res= c->to->wc_mb(c, c->to, &wc[i], (unsigned char*)d, (unsigned char*)d_e);
-      if (res > 0)
+    }
+    else if (res != UDM_CHARSET_CACHEDUNI)
+      goto outaway;
+
+    res= wc_mb(c, c->to, &wc[0], (unsigned char*)d, (unsigned char*)d_e);
+    if (res > 0)
+    {
+      d+= res;
+    }
+    else if (res == UDM_CHARSET_ILUNI)
+    {
+      if (c->flags & UDM_RECODE_HTML_NONASCII)
       {
-        d+= res;
-      }
-      else if (res == UDM_CHARSET_ILUNI && wc[i] != '?')
-      {
-        if (c->flags & UDM_RECODE_HTML_NONASCII)
+        if ( d_e - d > 8)
         {
-          if ( d_e - d > 8)
-          {
-            res = sprintf(d, "&#%d;", wc[i]);
-            d+= res;
-          }
-          else
-            break;
+          res= sprintf(d, "&#%d;", wc[0]);
+          d+= res;
         }
         else
-        {
-          wc[i] = '?';
-          goto outp;
-        }
+          break;
       }
       else
-        goto outaway;
+      {
+        wc[0]= '?';
+        res= wc_mb(c, c->to, &wc[0], (unsigned char*) d, (unsigned char*) d_e);
+        if (res <= 0)
+         goto outaway;
+      }
     }
+    else
+      goto outaway;
+      
+    UDM_ASSERT(wc[0] != 0 || s >= s_e);
   }
-
+  
 outaway:  
-  if(d < d_e)
-    res= c->to->wc_mb(c, c->to, &zero, (unsigned char*)d, (unsigned char*)d_e);
-  c->ibytes= s-s_o;
-  return (c->obytes= d - d_o);
+
+/*
+if (d - d_o  && d_o[0] != 0)
+  fprintf(stderr, "< %s %s %d '%.*s'\n", c->from->name, c->to->name, d - d_o, d - d_o, d_o);
+*/
+  return d - d_o;
 }
 
 __C_LINK const char * __UDMCALL UdmCsGroup(const UDM_CHARSET *cs)
