@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -151,6 +151,30 @@ size_t UdmDSTRAppend (UDM_DSTR *dstr, const char *data, size_t size_data) {
 	return(size_data);
 }
 
+
+static char hex_digits[]= "0123456789ABCDEF";
+
+/*
+  This function does not put trailing \x00 character.
+*/
+size_t
+UdmDSTRAppendHex(UDM_DSTR *dstr, const char *s, size_t slen)
+{
+  char *d;
+  size_t new_size= dstr->size_data + 2 * slen;
+  if (!slen || (UDM_OK != UdmDSTRRealloc(dstr, new_size)))
+    return 0;
+  for (d= dstr->data + dstr->size_data; slen; slen--, s++)
+  {
+    unsigned char ch= *((const unsigned char*) s);
+    *d++= hex_digits[(ch >> 4) & 0x0F];
+    *d++= hex_digits[ch & 0x0F];
+  }
+  dstr->size_data= new_size;
+  return dstr->size_data;
+}
+
+
 size_t UdmDSTRAppendSTR (UDM_DSTR *dstr, const char *data) {
 	return(UdmDSTRAppend(dstr, data, strlen(data)));
 }
@@ -406,27 +430,46 @@ __C_LINK size_t __UDMCALL udm_base64_encode (const char *src, char *store,
 }
 
 
-__C_LINK size_t __UDMCALL udm_base64_decode(char * dst, const char * src, size_t len){
-	int count=0;
-	int b[4];
-	char * dst0=dst;
-	
-	for( ; (*src)&&(len>3); src++){
-		char * p=strchr(base64,*src);
-		
-		b[count]=p?p-base64:0;
-		if(++count==4){
-			int res;
-			res=b[3]+b[2]*64+b[1]*64*64+b[0]*64*64*64;
-			*dst++=(res>>16)&0xFF;
-			*dst++=(res>>8)&0xFF;
-			*dst++= res&0xFF;
-			count=0;
-			len-=3;
-		}
-	}
-	*dst='\0';
-	return dst-dst0;
+static char from_base64[]=
+{
+/*00*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*10*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*20*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,62, 0, 0, 0,63, /*  !"#$%&'()*+,-./ */
+/*30*/  52,53,54,55,56,57,58,59,60,61, 0, 0, 0, 0, 0, 0, /* 0123456789:;<=>? */
+/*40*/   0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14, /* @ABCDEFGHIJKLMNO */
+/*50*/  15,16,17,18,19,20,21,22,23,24,25, 0, 0, 0, 0, 0, /* PQRSTUVWXYZ[\]^_ */
+/*60*/   0,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40, /* `abcdefghijklmno */
+/*70*/  41,42,43,44,45,46,47,48,49,50,51, 0, 0, 0, 0, 0, /* pqrstuvwxyz{|}~  */
+/*80*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*90*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*A0*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*B0*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*C0*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*D0*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*E0*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+/*F0*/   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+};
+
+
+__C_LINK size_t __UDMCALL
+udm_base64_decode(char *dst, const char *src, size_t len)
+{
+  char *dst0= dst;
+  
+  for( ; *src && len > 3 ; )
+  {
+    int b0= from_base64[(unsigned char) *src++];
+    int b1= from_base64[(unsigned char) *src++];
+    int b2= from_base64[(unsigned char) *src++];
+    int b3= from_base64[(unsigned char) *src++];
+    int res= b3 + b2*64 + b1*64*64 + b0*64*64*64;
+
+    *dst++= (res>>16)&0xFF;
+    *dst++= (res>>8)&0xFF;
+    *dst++= res&0xFF;
+  }
+  *dst= '\0';
+  return dst - dst0;
 }
 
 
@@ -868,6 +911,65 @@ static time_t ap_tm2sec(const struct tm * t)
 	return days;		/* must be a valid time */
 }
 
+
+/*
+  Scan date string: YYYY-MM-DD
+*/
+static void
+scan_yyyy_mm_dd(struct tm *ds, const char *date)
+{
+  ds->tm_year= ((date[0] - '0') * 10 + (date[1] - '0') - 19) * 100 +
+               (date[2] - '0') * 10 + date[3] - '0';
+  ds->tm_mon=  (date[5] - '0') * 10 + (date[6] - '0') - 1;
+  ds->tm_mday= (date[8] - '0') * 10 + (date[9] - '0');
+}
+
+
+/*
+  Scan year: YYYY
+*/
+static void
+scan_yyyy(struct tm *ds, const char *date)
+{
+  ds->tm_year= ((date[0] - '0') * 10 + (date[1] - '0') - 19) * 100 +
+               (date[2] - '0') * 10 + date[3] - '0';
+}
+
+
+/*
+  Scan time string:   hh:mm:ss
+*/
+static void
+scan_hh_mm_ss(struct tm *ds, const char *timstr)
+{
+  ds->tm_hour= ((timstr[0] - '0') * 10) + (timstr[1] - '0');
+  ds->tm_min= ((timstr[3] - '0') * 10) + (timstr[4] - '0');
+  ds->tm_sec= ((timstr[6] - '0') * 10) + (timstr[7] - '0');
+}
+
+static const int months[12] =
+{
+  ('J' << 16) | ('a' << 8) | 'n', ('F' << 16) | ('e' << 8) | 'b',
+  ('M' << 16) | ('a' << 8) | 'r', ('A' << 16) | ('p' << 8) | 'r',
+  ('M' << 16) | ('a' << 8) | 'y', ('J' << 16) | ('u' << 8) | 'n',
+  ('J' << 16) | ('u' << 8) | 'l', ('A' << 16) | ('u' << 8) | 'g',
+  ('S' << 16) | ('e' << 8) | 'p', ('O' << 16) | ('c' << 8) | 't',
+  ('N' << 16) | ('o' << 8) | 'v', ('D' << 16) | ('e' << 8) | 'c'
+};
+
+/*
+  Convert month name to month, 0..11
+*/
+static void
+scan_month_name(struct tm *ds, const char *monstr)
+{
+  int mon, mint= (monstr[0] << 16) | (monstr[1] << 8) | monstr[2];
+  for (mon= 0; mon < 12; mon++)
+    if (mint == months[mon])
+      break;
+  ds->tm_mon= mon;
+}
+
 /*
  * Parses an HTTP date in one of three standard forms:
  *
@@ -916,181 +1018,153 @@ static time_t ap_tm2sec(const struct tm * t)
  *
  */
 time_t UdmHttpDate2Time_t(const char *date){
-    struct tm ds;
-    int mint, mon;
-    const char *monstr, *timstr;
-    static const int months[12] =
-    {
-	('J' << 16) | ('a' << 8) | 'n', ('F' << 16) | ('e' << 8) | 'b',
-	('M' << 16) | ('a' << 8) | 'r', ('A' << 16) | ('p' << 8) | 'r',
-	('M' << 16) | ('a' << 8) | 'y', ('J' << 16) | ('u' << 8) | 'n',
-	('J' << 16) | ('u' << 8) | 'l', ('A' << 16) | ('u' << 8) | 'g',
-	('S' << 16) | ('e' << 8) | 'p', ('O' << 16) | ('c' << 8) | 't',
-	('N' << 16) | ('o' << 8) | 'v', ('D' << 16) | ('e' << 8) | 'c'};
+  struct tm ds;
 
-    if (!date)
-	return BAD_DATE;
-
-    while (*date && ap_isspace(*date))	/* Find first non-whitespace char */
-	++date;
-
-    if (*date == '\0')
-	return BAD_DATE;
-
-    if (ap_checkmask(date, "####-##-##"))
-    {
-      /*
-         2004-01-02 non-standard format,
-         but we can meet it quit often in user meta tags.
-      */
-      ds.tm_year= ((date[0] - '0') * 10 + (date[1] - '0') - 19) * 100 +
-                  (date[2] - '0') * 10 + date[3] - '0';
-      ds.tm_mon= (date[5] - '0') * 10 + (date[6] - '0') - 1;
-      ds.tm_mday= (date[8] - '0') * 10 + (date[9] - '0');
-      ds.tm_hour= 0;
-      ds.tm_min= 0;
-      ds.tm_sec= 0;
-      goto ret; 
-    }
-    else if (ap_checkmask(date, "##.##.####"))
-    {
-      /*
-         0123456789
-         02.01.2005 non-standard format,
-         but we can meet it quit often in
-         user meta tags in Germany.
-      */
-      ds.tm_year= ((date[6] - '0') * 10 + (date[7] - '0') - 19) * 100 +
-                  (date[8] - '0') * 10 + date[9] - '0';
-      ds.tm_mon= (date[3] - '0') * 10 + (date[4] - '0') - 1;
-      ds.tm_mday= (date[0] - '0') * 10 + (date[1] - '0');
-      ds.tm_hour= 0;
-      ds.tm_min= 0;
-      ds.tm_sec= 0;
-      goto ret; 
-    }
-
-    if ((date = strchr(date, ' ')) == NULL)	/* Find space after weekday */
-	return BAD_DATE;
+  if (!date)
+    return BAD_DATE;
 
+  while (*date && ap_isspace(*date))	/* Find first non-whitespace char */
     ++date;
+
+  if (*date == '\0')
+    return BAD_DATE;
+
+  if (ap_checkmask(date, "####-##-##"))
+  {
     /*
-      Now date points to first char after space, which should be
-      start of the actual date information for all formats.
+       2004-01-02 non-standard format,
+       but we can meet it quit often in user meta tags.
     */
-    if (ap_checkmask(date, "## @$$ #### ##:##:## *")) {
-        /*
-          RFC 1123 format
-          Sun, 06 Nov 1994 08:49:37 GMT
-        */
-	ds.tm_year = ((date[7] - '0') * 10 + (date[8] - '0') - 19) * 100;
-	if (ds.tm_year < 0)
-	    return BAD_DATE;
+    scan_yyyy_mm_dd(&ds, date);
+    ds.tm_hour= ds.tm_min= ds.tm_sec= 0;
+    goto check_valid_date;
+  }
+  else if (ap_checkmask(date, "##.##.####"))
+  {
+    /*
+       0123456789
+       02.01.2005 non-standard format,
+       but we can meet it quit often in
+       user meta tags in Germany.
+    */
+    scan_yyyy(&ds, date + 6);
+    ds.tm_mon= (date[3] - '0') * 10 + (date[4] - '0') - 1;
+    ds.tm_mday= (date[0] - '0') * 10 + (date[1] - '0');
+    ds.tm_hour= ds.tm_min= ds.tm_sec= 0;
+    goto check_valid_date;
+  }                         /* 01234567890123456789 */
+  else if (ap_checkmask(date, "####-##-##T##:##:##Z"))
+  {
+    /*
+      W3C format, also used by OpenSearch
+      http://www.w3.org/TR/NOTE-datetime
+      http://opensearch.a9.com/spec/1.1/response/
 
-	ds.tm_year += ((date[9] - '0') * 10) + (date[10] - '0');
+      Complete date plus hours, minutes, seconds
+      and UTC timezone designator
+      2005-01-02T10:20:30Z
+    */
+    scan_yyyy_mm_dd(&ds, date);
+    scan_hh_mm_ss(&ds, date + 11);
+    goto check_valid_time;
+  }
 
-	ds.tm_mday = ((date[0] - '0') * 10) + (date[1] - '0');
+  if ((date = strchr(date, ' ')) == NULL)	/* Find space after weekday */
+    return BAD_DATE;
 
-	monstr = date + 3;
-	timstr = date + 12;
-    }                          /*0123456789ABCDEFGHIJK*/
-    else if (ap_checkmask(date, "# @$$ #### ##:##:## *"))
-    {
-        /*
-          The same as above, but with a one-digit day
-          Sun, 6 Nov 1994 08:49:37 GMT
-        */
-	ds.tm_year = ((date[6] - '0') * 10 + (date[7] - '0') - 19) * 100;
-	if (ds.tm_year < 0)
-	    return BAD_DATE;
-	ds.tm_year += ((date[8] - '0') * 10) + (date[9] - '0');
-	ds.tm_mday = date[0] - '0';
-	monstr = date + 2;
-	timstr = date + 11;
-    }
-    else if (ap_checkmask(date, "##-@$$-## ##:##:## *")) {
-       /*
-         RFC 850 format:
-         Sunday, 06-Nov-94 08:49:37 GMT
-       */
-	ds.tm_year = ((date[7] - '0') * 10) + (date[8] - '0');
-	if (ds.tm_year < 70)
-	    ds.tm_year += 100;
+  ++date;
 
-	ds.tm_mday = ((date[0] - '0') * 10) + (date[1] - '0');
+  /*
+    Now date points to first char after space, which should be
+    start of the actual date information for all formats.
+  */
+  if (ap_checkmask(date, "## @$$ #### ##:##:## *"))
+  {
+    /*
+      RFC 1123 format
+      Sun, 06 Nov 1994 08:49:37 GMT
+    */
+    scan_yyyy(&ds, date + 7);
+    ds.tm_mday= ((date[0] - '0') * 10) + (date[1] - '0');
+    scan_month_name(&ds, date + 3);
+    scan_hh_mm_ss(&ds, date + 12);
+  }                          /*0123456789ABCDEFGHIJK*/
+  else if (ap_checkmask(date, "# @$$ #### ##:##:## *"))
+  {
+    /*
+     The same as above, but with a one-digit day
+     Sun, 6 Nov 1994 08:49:37 GMT
+    */
+    scan_yyyy(&ds, date + 6);
+    ds.tm_mday= date[0] - '0';
+    scan_month_name(&ds, date + 2);
+    scan_hh_mm_ss(&ds, date + 11);
+  }
+  else if (ap_checkmask(date, "##-@$$-## ##:##:## *"))
+  {
+    /*
+      RFC 850 format:
+      Sunday, 06-Nov-94 08:49:37 GMT
+    */
+    ds.tm_year= ((date[7] - '0') * 10) + (date[8] - '0');
+    if (ds.tm_year < 70)
+      ds.tm_year+= 100;
+    ds.tm_mday= ((date[0] - '0') * 10) + (date[1] - '0');
+    scan_month_name(&ds, date + 3);
+    scan_hh_mm_ss(&ds, date + 10);
+  }
+  else if (ap_checkmask(date, "@$$ ~# ##:##:## ####*"))
+  {
+    /*
+      ANSI C's asctime() format
+      Sun Nov  6 08:49:37 1994
+    */
+    scan_yyyy(&ds, date + 16);
+    ds.tm_mday= (date[4] == ' ') ? 0 : (date[4] - '0') * 10;
+    ds.tm_mday+= (date[5] - '0');
+    scan_month_name(&ds, date);
+    scan_hh_mm_ss(&ds, date + 7);
+  }
+  else
+    return BAD_DATE;
 
-	monstr = date + 3;
-	timstr = date + 10;
-    }
-    else if (ap_checkmask(date, "@$$ ~# ##:##:## ####*")) {
-        /*
-          ANSI C's asctime() format
-          Sun Nov  6 08:49:37 1994
-        */
-	ds.tm_year = ((date[16] - '0') * 10 + (date[17] - '0') - 19) * 100;
-	if (ds.tm_year < 0)
-	    return BAD_DATE;
+check_valid_time:
 
-	ds.tm_year += ((date[18] - '0') * 10) + (date[19] - '0');
+  if ((ds.tm_hour > 23) || (ds.tm_min > 59) || (ds.tm_sec > 61))
+    return BAD_DATE;
 
-	if (date[4] == ' ')
-	    ds.tm_mday = 0;
-	else
-	    ds.tm_mday = (date[4] - '0') * 10;
+check_valid_date:
 
-	ds.tm_mday += (date[5] - '0');
+  if (ds.tm_mday <= 0 || ds.tm_mday > 31)
+    return BAD_DATE;
 
-	monstr = date;
-	timstr = date + 7;
-    }
-    else
-	return BAD_DATE;
+  if (ds.tm_mon > 11)
+    return BAD_DATE;
 
-    if (ds.tm_mday <= 0 || ds.tm_mday > 31)
-	return BAD_DATE;
+  if ((ds.tm_mday == 31) &&
+      (ds.tm_mon == 3 || ds.tm_mon == 5 || ds.tm_mon == 8 || ds.tm_mon == 10))
+    return BAD_DATE;
 
-    ds.tm_hour = ((timstr[0] - '0') * 10) + (timstr[1] - '0');
-    ds.tm_min = ((timstr[3] - '0') * 10) + (timstr[4] - '0');
-    ds.tm_sec = ((timstr[6] - '0') * 10) + (timstr[7] - '0');
+  /* February gets special check for leapyear */
 
-    if ((ds.tm_hour > 23) || (ds.tm_min > 59) || (ds.tm_sec > 61))
-	return BAD_DATE;
-
-    mint = (monstr[0] << 16) | (monstr[1] << 8) | monstr[2];
-    for (mon = 0; mon < 12; mon++)
-	if (mint == months[mon])
-	    break;
-    if (mon == 12)
-	return BAD_DATE;
-
-    if ((ds.tm_mday == 31) && (mon == 3 || mon == 5 || mon == 8 || mon == 10))
-	return BAD_DATE;
-
-    /* February gets special check for leapyear */
-
-    if ((mon == 1) &&
-	((ds.tm_mday > 29)
-	 || ((ds.tm_mday == 29)
-	     && ((ds.tm_year & 3)
-		 || (((ds.tm_year % 100) == 0)
-		     && (((ds.tm_year % 400) != 100)))))))
-	return BAD_DATE;
-
-    ds.tm_mon = mon;
-
-ret:
+  if ((ds.tm_mon == 1) &&
+      ((ds.tm_mday > 29)
+      || ((ds.tm_mday == 29)
+       && ((ds.tm_year & 3)
+      || (((ds.tm_year % 100) == 0)
+       && (((ds.tm_year % 400) != 100)))))))
+    return BAD_DATE;
 
 #if 0
-    {
-      int res= ap_tm2sec(&ds);
-      char buf[128];
-      strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &ds);
-      printf("parseHTTPdate: '%s' is %lu: strftime=%s, month=%d\n", date, res, buf, ds.tm_mon);
-      
-    }
+  {
+    int res= ap_tm2sec(&ds);
+    char buf[128];
+    strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &ds);
+    printf("parseHTTPdate: '%s' is %lu: strftime=%s, month=%d\n", date, res, buf, ds.tm_mon);
+  }
 #endif
 
-    return ap_tm2sec(&ds);
+  return ap_tm2sec(&ds);
 }
 
 /********
@@ -1894,34 +1968,41 @@ char * UdmStrRemoveDoubleChars(char * str, const char * sep){
 }
 
 
-void UdmUniRemoveDoubleSpaces(int * ustr){
-	int * u, *e, addspace=0;
-				
-	for(u=e=ustr;*u;){
-		switch(*u){
-			case ' ':
-			case '\t':
-			case '\r':
-			case '\n':
-			case 0xA0: /* nbsp */
-				addspace=1;
-				u++;
-				break;
-			default:
-				if(addspace){
-					if(e>ustr){
-						*e=' ';
-						e++;
-					}
-					addspace=0;
-				}
-				*e=*u;
-				e++;
-				u++;
-				break;
-		}
-	}
-	*e=0;
+size_t UdmUniRemoveDoubleSpaces(int *ustr)
+{
+  int *u, *e, addspace=0;
+  
+  for(u= e= ustr; *u ;)
+  {
+    switch(*u)
+    {
+      case ' ':
+      case '\t':
+      case '\r':
+      case '\n':
+      case 0xA0: /* nbsp */
+        addspace=1;
+        u++;
+        break;
+      
+      default:
+        if(addspace)
+        {
+          if(e > ustr)
+          {
+            *e=' ';
+            e++;
+          }
+          addspace= 0;
+        }
+        *e= *u;
+        e++;
+        u++;
+        break;
+    }
+  }
+  *e= 0;
+  return e - ustr;
 }
 
 void UdmUniPrint(int * ustr){
@@ -2259,56 +2340,6 @@ void UdmViolationExit(void *v) {
 
 #endif /* USE_PARANOIA */
 
-const char udm_soundex[]= "01230120022455012623010202";
-
-static char udm_soundex_code(UDM_CHARSET *cset, const char *ptr)
-{
-  unsigned char ch= toupper((unsigned char)*ptr);
-  if (ch < 'A' || ch > 'Z')
-  {
-    if (isalpha(ch))
-      return '0';
-    return 0;
-  }
-  return(udm_soundex[ch - 'A']);
-}
-
-
-void UdmSoundex(UDM_CHARSET *cs, char *dst, const char *src, size_t srclen)
-{
-  char ch, prev, *end, *padend, *dst0= dst;
-  const char *srcend= src + srclen;
-
-  while (!isalpha((unsigned char)src[0]) && srclen > 0)
-  {
-    src++;
-    srclen--;
-  }
-
-  *dst++= toupper((unsigned char)*src);
-  prev= udm_soundex_code(cs, src);
-  end= dst + 24;
-  padend= dst + 3;
-
-  src++;
-  for ( ; dst < end && src < srcend; src++)
-  {
-    ch= udm_soundex_code(cs, src);
-    if (!isalpha((unsigned char)src[0]))
-      continue;
-    if ((ch != '0') && (ch != prev)) /* if not skipped or double */
-    {
-      *dst++ = ch;
-      prev = ch;
-    }
-  }
-
-  while (dst < padend)
-   *dst++ = '0';
-
-  *dst=0;
-}
-
 
 /*
  * Note that buf must be at least X * 5 + 5 bytes. Where X is
@@ -2509,3 +2540,18 @@ int udm_dezint4(const char *buf, int4 *array, int buf_len)
   return(array_end - array);
 }
 
+#ifdef HAVE_DEBUG
+void UdmSort(void *base, size_t nmemb, size_t size,
+             int(*compar)(const void *, const void *))
+{
+  UDM_ASSERT(nmemb > 0);
+  qsort(base, nmemb, size, compar);
+}
+
+void *UdmBSearch(const void *key, const void *base, size_t nmemb,
+                 size_t size, int (*compar)(const void *, const void *))
+{
+  UDM_ASSERT(nmemb > 0);
+  return bsearch(key, base, nmemb, size, compar);
+}
+#endif
