@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+/* Copyright (C) 2000-2006 Lavtech.com corp. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,8 +25,8 @@
 
 #include "udm_common.h"
 #include "udm_wild.h"
+#include "udm_utils.h"
 
-#define udm_tolower(c) (tolower(((unsigned char)(c))))
 
 int UdmWildCmp(const char *str, const char *wexp)
 {
@@ -48,22 +48,27 @@ int UdmWildCmp(const char *str, const char *wexp)
     return (str[x] != '\0');
 }
 
+
 int __UDMCALL UdmWildCaseCmp(const char *str, const char *wexp)
 {
-    int x, y;
-
-    for (x = 0, y = 0; wexp[y]; ++y, ++x) {
-	if ((!str[x]) && (wexp[y] != '*'))return -1;
-	if (wexp[y] == '*') {
-	    while (wexp[++y] == '*');
-	    if (!wexp[y])return 0;
-	    while (str[x]) {
-		int ret;
-		if ((ret = UdmWildCaseCmp(&str[x++], &wexp[y])) != 1)return ret;
-	    }
-	    return -1;
-	}else
-	if ((wexp[y] != '?') && (udm_tolower(str[x]) != udm_tolower(wexp[y])))return 1;
+  for ( ; *wexp; ++str, ++wexp)
+  {
+    if (!*str && *wexp != '*')return -1;
+    if (*wexp == '*')
+    {
+      while (*++wexp == '*');
+      if (!*wexp)return 0;
+      while (*str)
+      {
+        int ret;
+        if ((ret= UdmWildCaseCmp(str++, wexp)) != 1)return ret;
+      }
+      return -1;
     }
-    return (str[x] != '\0');
+    else if ((*wexp != '?') &&
+             (udm_l1tolower[(unsigned char)*str] !=
+              udm_l1tolower[(unsigned char)*wexp]))
+      return 1;
+  }
+  return (*str != '\0');
 }
